Attribute VB_Name = "ControlSM"
'  FILE:       ControlSM.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains all the control flow procedures for the
'              project viz. enabling menu options depending on
'              certain conditions, displaying a context-sensitive
'              menu, etc.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
' This module is called ControlSM, since a Control is a standard
' Visual Basic object and we want to avoid any confusion with it.

Option Explicit

Private Const mstrModuleName As String = "ControlSM."

' The background colour used to indicate that a field has
' been disabled
Private Const mstrKeywordBackground = &HE0E0E0

' Strings used in the popup menu
Private Const mstrCreate = "&Create"
Private Const mstrCreateManager = "Create &Manager"

Private Const mstrDirty As String = "Dirty"
Private Const mstrClean As String = "Clean"

Private Const MAX_APP_EXTENSIONS As Integer = 10
Private Sub EnablePaste(bEnabled As Boolean)

    frmMain.mnuEditPaste.Enabled = bEnabled
    frmMain.mnuPopupPaste.Enabled = bEnabled
    frmMain.tbToolBar.Buttons(tbPaste).Enabled = bEnabled
    
End Sub
Public Sub EnableRun(bEnabled As Boolean)
    On Error GoTo EnableRunErr
    
    ' Run and Stop r mutually exclusive operations
    frmMain.mnuFileRun.Enabled = bEnabled
    frmMain.tbToolBar.Buttons(tbRun).Enabled = bEnabled
    
    frmMain.mnuFileStop.Enabled = Not bEnabled
    frmMain.tbToolBar.Buttons(tbStop).Enabled = Not bEnabled
    
    Exit Sub
    
EnableRunErr:
    LogErrors Errors
    Resume Next
    
End Sub


Private Sub SetPropertiesEnabled(bEnabled As Boolean)
    
    On Error GoTo SetPropertiesEnabledErr
    
    frmMain.mnuFileProperties.Enabled = bEnabled
    frmMain.mnuPopupProperties.Enabled = bEnabled
    frmMain.tbToolBar.Buttons(tbProperties).Enabled = bEnabled
    
    Exit Sub

SetPropertiesEnabledErr:
    ' Log any error messages
    LogErrors Errors
    Resume Next
    
End Sub
Private Sub SetDeleteEnabled(bEnabled As Boolean)
    
    On Error GoTo SetDeleteEnabledErr
    
    frmMain.mnuFileDelete.Enabled = bEnabled
    frmMain.mnuPopupDelete.Enabled = bEnabled
    frmMain.tbToolBar.Buttons(tbDelete).Enabled = bEnabled
    
    Exit Sub

SetDeleteEnabledErr:
    ' Log any error messages
    LogErrors Errors
    Resume Next
    
End Sub

Public Sub ToggleCutAndPaste(bEnabled As Boolean)

    'The edit and paste options are complementary
    SetEditOptions (Not bEnabled)
    EnablePaste bEnabled
    
End Sub
Public Sub HideControlsInFrame(frmForm As Form, _
        fraToHide As Frame)
    ' Hides all the controls on the passed in frame. Recursive
    ' procedure that calls itself if it finds a frame within
    ' the frame being processed.
    
    Dim ctrNext As Control
    
    ' Hides all the fields on the passed in container frame
    On Error GoTo HideControlsInFrameErr
    
    For Each ctrNext In frmForm.Controls
        If TypeName(ctrNext) <> "ImageList" Then
            If ctrNext.Container Is fraToHide Then
                If TypeOf ctrNext Is Frame Then
                    Call HideControlsInFrame(frmForm, ctrNext)
                Else
                    ctrNext.Visible = False
                End If
            End If
        End If
    Next ctrNext
    fraToHide.Visible = False
    
    Exit Sub

HideControlsInFrameErr:
    ' Log the VB error
    Call LogErrors(Errors)
    
    ' Non-critical - continue
    Resume Next

End Sub

Public Sub HideIteratorFields()
    ' Hides all the fields needed to create iterators on the
    ' Step Properties screen
    On Error GoTo HideIteratorFieldsErr
    
    Call HideControlsInFrame(frmSteps, frmSteps.fraIterator)
    
    Exit Sub

HideIteratorFieldsErr:
    ' Log the VB error
    Call LogErrors(Errors)
    
    ' Non-critical - continue
    Resume Next

End Sub

Public Sub CopyTextToGrid(MSGridCtrl As MSFlexGrid, _
        Edt As TextBox)
    ' Copies the contents of the text box to the grid
    
    On Error GoTo CopyTextToGridErr
    
    If Edt.Visible = False Then
        Exit Sub
    End If
    
    MSGridCtrl.Text = Edt.Text
    Edt.Visible = False
    
    Exit Sub
    
CopyTextToGridErr:
    Call LogErrors(Errors)

End Sub
Sub EditKeyCode(MSFlexGrid As Control, Edt As Control, _
        KeyCode As Integer, Shift As Integer)

    On Error GoTo MSFlexGridEditErr
    
    ' Standard edit control processing.
    Select Case KeyCode
    
        Case vbKeyEscape ' ESC: hide, return focus to MSFlexGrid.
            Edt.Visible = False
            MSFlexGrid.SetFocus
        
        Case vbKeyReturn  ' ENTER return focus to MSFlexGrid.
            MSFlexGrid.SetFocus
        
        Case vbKeyUp     ' Up.
            MSFlexGrid.SetFocus
            DoEvents
            If MSFlexGrid.Row > MSFlexGrid.FixedRows Then
                MSFlexGrid.Row = MSFlexGrid.Row - 1
            End If
            
        Case vbKeyDown        ' Down.
            MSFlexGrid.SetFocus
            DoEvents
            If MSFlexGrid.Row >= MSFlexGrid.Rows - 1 Then
                MSFlexGrid.Rows = MSFlexGrid.Rows + 1
            End If
            MSFlexGrid.Row = MSFlexGrid.Row + 1
    End Select
    
    Exit Sub
    
MSFlexGridEditErr:
    Call LogErrors(Errors)

End Sub
Sub MSFlexGridEdit(MSFlexGrid As Control, Edt As Control, _
        KeyAscii As Integer)
    
    On Error GoTo MSFlexGridEditErr
    
    ' Use the character that was typed.
    Select Case KeyAscii
    
        ' A space means edit the current text.
        Case 0 To 32
            Edt = MSFlexGrid
            Edt.SelStart = 1000
        
        ' Anything else means replace the current text.
        Case Else
            Edt = Chr(KeyAscii)
            Edt.SelStart = 1
    End Select
    
    ' Show Edt at the right place.
    Edt.Move MSFlexGrid.Left + MSFlexGrid.CellLeft, _
            MSFlexGrid.Top + MSFlexGrid.CellTop, _
            MSFlexGrid.CellWidth, MSFlexGrid.CellHeight
    Edt.Visible = True
    
    ' And let it work.
    Edt.SetFocus
    
    Exit Sub
    
MSFlexGridEditErr:
    Call LogErrors(Errors)

End Sub
Private Sub ListEdit(ListToEdit As ListBox, Edt As TextBox, _
        KeyAscii As Integer, EditX As Long, EditY As Long, _
        CurrentValue As String)
    
    On Error GoTo ListEditErr
    
    ' Use the character that was typed.
    Select Case KeyAscii
        ' A return means the edit is complete
        Case vbKeyReturn
            'Call CopyTextToList(ListToEdit, Edt)
    
        ' A space means edit the current text.
        Case 0 To 32
            Edt = CurrentValue
            Edt.SelStart = 1000
        
        ' Anything else means replace the current text.
        Case Else
            Edt = Chr(KeyAscii)
            Edt.SelStart = 1
    End Select
    
    ' Show Edt at the right place.
    Edt.Move EditY, EditX
    Edt.Visible = True
    
    ' And let it work.
    Edt.SetFocus
    
    Exit Sub
    
ListEditErr:
    Call LogErrors(Errors)

End Sub
Private Sub EnableTextEntry(txtCtl As TextBox, _
        blnEnabled As Boolean)
    ' Enables/disables edits on the passed in text box
    ' based on the value of the enabled flag

    On Error GoTo EnableTextEntryErr
    
    txtCtl.Locked = Not blnEnabled
    txtCtl.TabStop = blnEnabled
'    txtCtl.Enabled = blnEnabled
    
    Exit Sub

EnableTextEntryErr:
    LogErrors Errors
    Resume Next

End Sub
Private Sub EnablePreview(blnEnabled As Boolean)
    ' Enables/disables the Browse and Preview commands
    ' based on the value of the enabled flag

    On Error GoTo EnablePreviewErr
    
    frmSteps.cmdPreview.Enabled = blnEnabled
    frmSteps.cmdOpen.Enabled = blnEnabled
    
    Exit Sub

EnablePreviewErr:
    LogErrors Errors
    Resume Next

End Sub

Public Sub EnteringStepFile()

    On Error Resume Next

    If frmSteps.txtStepTextFileName.Tag = gstrEntry Then
        If StringEmpty(frmSteps.txtStepTextFileName) Then
            ' The user has deleted a previously entered file name,
            ' if any. Switch the user out from the File Preview mode to
            ' allow him to enter the step text directly at this point
            Call ResetAppKeywordColours(frmSteps.txtStepText)
            Call EnableTextEntry(frmSteps.txtStepText, True)
            frmSteps.txtStepText.Tag = gstrEntry
            
            ' Also remove the preview contents being displayed, if any
            frmSteps.txtStepText.Text = gstrEmptyString
            frmSteps.lblStepText.Caption = "The &Step to be Executed"
            
            ' Cannot preview or open a zero-length file name!
            Call EnablePreview(False)
        Else
            ' Since the user is entering a file name, disable
            ' the step text field
            Call ResetAppKeywordColours(frmSteps.txtStepTextFileName)
            Call SetAppKeywordColours(frmSteps.txtStepText)
            Call EnableTextEntry(frmSteps.txtStepText, False)
            
            ' Enable the preview or open commands
            Call EnablePreview(True)
        End If
    End If
    
End Sub
Public Sub EnteringStepText()

    On Error GoTo EnteringStepTextErr

    ' Make sure that the event is being triggered by the
    ' user actually entering text (the event will be triggered
    ' even when the text property is set to s'thing during
    ' a file preview)
    If frmSteps.txtStepText.Tag = gstrEntry Then
        If StringEmpty(frmSteps.txtStepText) Then
            ' The user has deleted previously entered step text,
            ' if any. Allow him to enter a step file.
            Call ResetAppKeywordColours(frmSteps.txtStepTextFileName)
            frmSteps.txtStepTextFileName.Tag = gstrEntry
            frmSteps.txtStepTextFileName.Text = gstrEmptyString
            Call FileTextEnabled(True)
        Else
            ' Since the user is entering text, disable all the
            ' file name fields
            Call SetKeywordText(frmSteps.txtStepTextFileName, _
                    gstrInline, gstrInline)
            Call ResetAppKeywordColours(frmSteps.txtStepText)
            Call FileTextEnabled(False)
        End If
    End If
    Exit Sub

EnteringStepTextErr:
    LogErrors Errors
    Resume Next

End Sub
Public Sub ContextSensitiveMenus(ByVal nodSelected As comctllib.Node)
    
    ' Disables/enables the different toolbar, menu and context-
    ' sensitive options based on the type of node selected
    ' by the user
    
    Dim strKey As String
    Dim intNodeType As Integer
    
    On Error GoTo ContextSensitiveMenusErr
    
    strKey = nodSelected.Key
    intNodeType = TypeOfNode(strKey)
    
    If intNodeType = gintWorkspace Then
        ' If the user is on a workspace node, then disable all
        ' edit operations
        Call SetEditOptions(False)
        Call SetDataOptions(True)
        EnablePaste False
    ElseIf intNodeType = gintNodeParamBuiltIn Or IsBuiltInLabel(strKey) Then
        ' If the user is on a workspace node, then disable all
        ' edit operations
        Call SetEditOptions(False)
        
        If intNodeType = gintNodeParamBuiltIn Then
            Call SetPropertiesEnabled(True)
            Call SetDeleteEnabled(False)
        Else
            Call SetDataOptions(False)
        End If
                
        EnablePaste False
        
    ElseIf IsLabel(strKey) Then
        ' If the user is on a label node, then disable all
        ' edit and other data-specific operations
        Call SetDataOptions(False)
        Call SetEditOptions(False)
        
        If frmMain.DuringEdit() Then
            EnablePaste True
        Else
            EnablePaste False
        End If
    Else
        ' Enable all the options since the user is on a parameter
        ' or a step node
        Call SetDataOptions(True)
            
        If frmMain.DuringEdit() Then
            ToggleCutAndPaste True
        Else
            ToggleCutAndPaste False
        End If
    End If
    
    StepSpecificOptions (IsManager(StepKey:=strKey))
    
    ' Call a procedure to modify the create options depending
    ' on the type
    Call ModifyCreateMenu(intNodeType)
    
    Exit Sub
    
ContextSensitiveMenusErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "ContextSensitiveMenus"
    ShowError errUnableToModifyMenu
    
End Sub
Public Sub EnableConstraintCommands(lstGlobals As ListBox, _
        lstConstraints As ListBox, _
        tblCommands As Toolbar)

    On Error GoTo EnableConstraintCommandsErr
    
    ' The only operations permitted on a constraint are
    ' move up, move down and delete operations
    
    tblCommands.Buttons(gintAdd).Enabled = False
    If lstConstraints.ListIndex > -1 Then
        ' Enable the constraint commands only if a constraint
        ' has been selected by the user
        tblCommands.Buttons(gintDelete).Enabled = True
        
        If lstConstraints.ListIndex - 1 >= 0 Then
            ' A Move Up operation is valid - enable the command
            tblCommands.Buttons(gintMoveUp).Enabled = True
        Else
            tblCommands.Buttons(gintMoveUp).Enabled = False
        End If
        If lstConstraints.ListIndex + 1 < _
                lstConstraints.ListCount Then
            ' A Move Down operation is valid - enable the command
            tblCommands.Buttons(gintMoveDown).Enabled = True
        Else
            tblCommands.Buttons(gintMoveDown).Enabled = False
        End If
    End If

    Exit Sub
    
EnableConstraintCommandsErr:

    ' Log the VB error code and display a more friendly
    ' error message to the user
    Call LogErrors(Errors)
    ' Resume
    Resume Next
    
End Sub
Public Sub ModifyCreateMenu(ByVal intNodeType As Integer)
    ' Displays the Manager/Worker sub-menu if the user is
    ' on a Manager step or the Steps label
    
    On Error GoTo ModifyCreateMenuErr
    
    ' Set three properties in each of the conditions in the
    ' loop below -
    ' 1. The caption of the 0th element
    ' 2. Enabled property of the 0th element
    ' 3. Visible property of the Create Worker element
    If intNodeType = gintManagerStep Or intNodeType = gintStepLabel Then
        ' Display the Manager/Worker submenu
        frmMain.mnuPopupNew(0).Caption = mstrCreateManager
        frmMain.mnuPopupNew(0).Enabled = True
        
        frmMain.mnuPopupNew(gintWorkerStep).Visible = True
    ElseIf intNodeType = gintWorkerStep Then
        ' The user cannot create anything as a sub-step of a
        ' worker step - hide the create menu
        frmMain.mnuPopupNew(0).Caption = mstrCreate
        frmMain.mnuPopupNew(0).Enabled = False
        
        ' Hide the Create Worker option
        frmMain.mnuPopupNew(gintWorkerStep).Visible = False
    ElseIf intNodeType = gintNodeParamBuiltIn Or intNodeType = gintParamBuiltInLabel Then
        frmMain.mnuPopupNew(0).Caption = mstrCreate
        frmMain.mnuPopupNew(0).Enabled = False
        
        frmMain.mnuPopupNew(gintWorkerStep).Visible = False
    Else
        ' Let the user create an item
        frmMain.mnuPopupNew(0).Caption = mstrCreate
        frmMain.mnuPopupNew(0).Enabled = True
        
        ' But hide the Create Worker option!
        frmMain.mnuPopupNew(gintWorkerStep).Visible = False
    End If

    Exit Sub
    
ModifyCreateMenuErr:

    ' Log the VB error code
    Call LogErrors(Errors)
    Resume Next

End Sub

Private Sub FileTextEnabled(blnEnabled As Boolean)
    ' Sets the enabled property for the step text file name
    ' field, the preview, open and browse command buttons to
    ' the passed in value - these four controls go together
    ' as far as enabling them is concerned!
    On Error GoTo FileTextEnabledErr
    
    frmSteps.txtStepTextFileName.Enabled = blnEnabled
    frmSteps.cmdBrowseStepText.Enabled = blnEnabled
    frmSteps.cmdPreview.Enabled = blnEnabled
    frmSteps.cmdOpen.Enabled = blnEnabled
    
    Exit Sub

FileTextEnabledErr:
    ' Log the VB error
    Call LogErrors(Errors)
    
    ' Non-critical - continue
    Resume Next
    
End Sub

Private Sub SetAppKeywordColours(ctrCtrl As TextBox)

    On Error GoTo SetAppKeywordColoursErr

    ctrCtrl.BackColor = mstrKeywordBackground
    
    Exit Sub

SetAppKeywordColoursErr:
    LogErrors Errors
    Resume Next

End Sub
Private Sub ResetAppKeywordColours(ctrCtrl As TextBox)

    On Error GoTo ResetAppKeywordColoursErr

    ctrCtrl.BackColor = vbWindowBackground
    
    Exit Sub

ResetAppKeywordColoursErr:
    LogErrors Errors
    Resume Next

End Sub

Private Sub SetEditOptions(blnEnabled As Boolean)

    ' Disable/enables the options to cut and copy a node
    ' from the main menu, the context-sensitive menu and the
    ' toolbar based on the value of the passed in parameter
    
    On Error GoTo SetEditOptionsErr
    
    frmMain.mnuEditCut.Enabled = blnEnabled
    frmMain.mnuPopupCut.Enabled = blnEnabled
    frmMain.tbToolBar.Buttons(tbCut).Enabled = blnEnabled
    
    frmMain.mnuEditCopy.Enabled = blnEnabled
    frmMain.mnuPopupCopy.Enabled = blnEnabled
    frmMain.tbToolBar.Buttons(tbCopy).Enabled = blnEnabled
    
    Exit Sub
    
SetEditOptionsErr:
    
    ' Log any error messages
    LogErrors Errors
    Resume Next
    
End Sub
Private Sub SetDataOptions(blnEnabled As Boolean)

    ' Disable/enables all the Delete and Properties options -  from the main menu,
    ' the context-sensitive menu and the toolbar
    Call SetPropertiesEnabled(blnEnabled)
   
    Call SetDeleteEnabled(blnEnabled)
    
End Sub
Private Sub StepSpecificOptions(blnEnabled As Boolean)

    ' Disable/enables all options for operations that are only valid for step nodes,
    ' currently, Enable All and Disable All
    ' valid for labels - basically the edit operations and the
    ' Delete and Properties options -  from the main menu,
    ' the context-sensitive menu and the toolbar

    On Error GoTo StepSpecificOptionsErr
    
    frmMain.mnuEditEnableAll.Enabled = blnEnabled
    frmMain.mnuPopupEnableAll.Enabled = blnEnabled
    
    frmMain.mnuEditDisableAll.Enabled = blnEnabled
    frmMain.mnuPopupDisableAll.Enabled = blnEnabled
    
    Exit Sub
    
StepSpecificOptionsErr:
    
    ' Log any error messages
    LogErrors Errors
    Resume Next
    
End Sub

Public Sub SetStepTabOrder()

    On Error GoTo SetStepTabOrderErr
    
    ' Handles the tab order on the Step Properties screen
    ' Focus may shift to a control on an inactive tab, when
    ' one of the tabs has a container control (frame in this case)
    ' The solution is to disable all controls that are not on
    ' the active tab
    ' Also ensure that the command buttons Ok, Apply and Cancel,
    ' appear last in the taborder
    
    ' This solution was picked up from the Microsoft Knowledge Base
    ' article :
    ' http://support.microsoft.com/support/kb/articles/q178/4/69.asp
    
    Dim ctrTemp As Control
    
    For Each ctrTemp In frmSteps.Controls
        If TypeName(ctrTemp) <> "ImageList" Then
            If (ctrTemp.Container Is frmSteps.sstStepProperties) Then
                    ctrTemp.Enabled = (ctrTemp.Left >= 0)
            End If
        End If
    Next

    ' NOTE: When a control contained in an SSTab control is on an
    ' inactive tab, the Left property of the control is negative.
    
    Select Case frmSteps.sstStepProperties.Tab
        Case gintDefinition
            frmSteps.tblConstraintCommands.Visible = False
            
        Case gintExecution
            frmSteps.tblConstraintCommands.Visible = False
            
        Case gintMgrDefinition
            frmSteps.tblConstraintCommands.Visible = False
            
        Case gintPreExecutionSteps
            frmSteps.tblConstraintCommands.Visible = True
        
        Case gintPostExecutionSteps
            frmSteps.tblConstraintCommands.Visible = True
        
        Case gintFileLocations
            frmSteps.tblConstraintCommands.Visible = False
        
    End Select
    
    Exit Sub

SetStepTabOrderErr:
    ' Since the procedure is not critical, just log the error
    ' and exit
    Call LogErrors(Errors)
    Call WriteError(errSetTabOrderFailed, gstrSource, ctrTemp.Name)
    Resume Next
    
End Sub

Public Sub CheckParentValid(intNodeType As Integer)
    
    Dim strParentNodeKey As String
    
    On Error GoTo CheckParentValidErr

    If intNodeType = gintManagerStep Or intNodeType = gintWorkerStep Then
        ' If a worker or manager step is being created as a sub-step
        ' of another step, make sure that the parent is a manager
        ' step
        strParentNodeKey = ItemSelectedInTree(frmMain.treStepView)
        If Not StringEmpty(strParentNodeKey) Then
            If IsWorker(StepKey:=strParentNodeKey) Then
                ShowError errParentStepManager
                On Error GoTo 0
                Err.Raise vbObjectError + errParentStepManager, _
                        gstrSource, _
                        LoadResString(errParentStepManager)
            End If
        End If
    End If
    
    Exit Sub
    
CheckParentValidErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "CheckParentValid"
    On Error GoTo 0
    Err.Raise vbObjectError + errCheckParentValidFailed, _
            gstrSource, _
            LoadResString(errCheckParentValidFailed)
    
End Sub

Public Sub DisplayPopup()

    ' Displays the context-sensitive menu to the user
    ' Enables/disables options based on the node selected
    ' by the user
    
    On Error GoTo DisplayPopupErr
    
    frmMain.PopupMenu frmMain.mnuPopup
    
    Exit Sub
    
DisplayPopupErr:

    ' Log the VB error code and display a more friendly
    ' error message to the user
    Call LogErrors(Errors)
    Call ShowError(errDisplayPopupFailed)
    
End Sub
Public Sub DisplayProperties(Optional ByVal strNodeKey As String = gstrEmptyString)

    Dim lngNodeIdentifier As Long

    On Error GoTo DisplayPropertiesErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If StringEmpty(strNodeKey) Then
        strNodeKey = ItemSelectedInTree(frmMain.treStepView)
    End If
    lngNodeIdentifier = MakeIdentifierValid(strNodeKey)
    
    If IsInstanceNode(strNodeKey) Then
        Dim cStepRec As cStep
        Set cStepRec = gcSteps.QueryStep(lngNodeIdentifier)
        If cStepRec.StepType = gintManagerStep Then
            Call ShowModifyManagerStepForm(lngNodeIdentifier)
        Else
            Call ShowModifyWorkerStepForm(lngNodeIdentifier)
        End If
        Set cStepRec = Nothing
    Else
        Select Case TypeOfNode(strNodeKey)
            Case gintWorkspace
                Call ShowWorkspaceModify(lngNodeIdentifier)
            Case gintParameter, gintNodeParamExtension, gintNodeParamBuiltIn
                Call ShowModifyParameterForm(lngNodeIdentifier)
            Case gintNodeParamConnection
                Call ShowModifyConnectionForm(lngNodeIdentifier)
            Case gintConnectionDtl
                Call ShowModifyConnDtlForm(lngNodeIdentifier)
            Case gintGlobalStep
                Call ShowModifyGlobalStepForm(lngNodeIdentifier)
            Case gintManagerStep
                Call ShowModifyManagerStepForm(lngNodeIdentifier)
            Case gintWorkerStep
                Call ShowModifyWorkerStepForm(lngNodeIdentifier)
            Case Else
                BugMessage "Invalid node type detected by DisplayProperties!"
        End Select
    End If

    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
DisplayPropertiesErr:

    ' Log the VB error code and display a more friendly
    ' error message to the user
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "DisplayProperties"
    ' Reset the mousepointer
    Call ShowFree
    Call ShowError(errDisplayPropertiesFailed)
    
End Sub
Public Function TypeOfNode(Optional ByVal strNodeKey As String = gstrEmptyString) _
    As Integer
    ' If the node key is passed in, the function returns a constant
    ' indicating the type of node corresponding to the key
    ' Else, it returns a constant indicating the type of node
    ' selected by user in the tree view control
    
    Dim strSelectedNodeKey As String
    Dim lngMatch As Long
    Dim intLabelIdentifier As Integer

    On Error GoTo TypeOfNodeErr
    
    ' Sets the node key either to the passed in string, if specified,
    ' or the key for the node selected by the user
    If strNodeKey = gstrEmptyString Then
        ' If the node key was not passed in, call a function to
        ' see if s'thing was selected by the user
        strSelectedNodeKey = ItemSelectedInTree(frmMain.treStepView)
        If strSelectedNodeKey = gstrEmptyString Then
            ' If no selection has been made by the user,
            ' default processing will be for the workspace
            TypeOfNode = gintWorkspace
            Exit Function
        End If
    Else
        strSelectedNodeKey = strNodeKey
    End If
            
    ' Uses the node key to determine the type of node
    Select Case Left(strSelectedNodeKey, 1)
        Case mstrWorkspacePrefix
            TypeOfNode = gintWorkspace
        Case mstrParameterPrefix
            TypeOfNode = gintParameter
        Case mstrParamConnectionPrefix
            TypeOfNode = gintNodeParamConnection
        Case mstrConnectionDtlPrefix
            TypeOfNode = gintConnectionDtl
        Case mstrParamExtensionPrefix
            TypeOfNode = gintNodeParamExtension
        Case mstrParamBuiltInPrefix
            TypeOfNode = gintNodeParamBuiltIn
        Case gstrGlobalStepPrefix
            TypeOfNode = gintGlobalStep
        Case gstrManagerStepPrefix
            TypeOfNode = gintManagerStep
        Case gstrWorkerStepPrefix
            TypeOfNode = gintWorkerStep
        Case Else
            ' Check if the node is a label
            ' For a label node, the type returned will be the
            ' corresponding object type e.g. if the user selects
            ' the "Parameters" node, we return gintParameter
            lngMatch = InStr(strSelectedNodeKey, mstrLabelPrefix)
            BugAssert lngMatch > 0
            If lngMatch > 0 Then
                ' For all labels, determine the node type
                intLabelIdentifier = CInt(Mid(strSelectedNodeKey, lngMatch + 1))
                TypeOfNode = intLabelIdentifier
            End If
    End Select
        
    Exit Function
    
TypeOfNodeErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errTypeOfNodeFailed, _
            mstrModuleName & "TypeOfNode", _
            LoadResString(errTypeOfNodeFailed)
        
End Function

Sub NewNode(ByVal intNodeType As Integer, Optional cStepRec As cStep = Nothing)

    Dim lngWorkspaceId As Long
    
    On Error GoTo NewNodeErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If intNodeType <> gintWorkspace Then
        If Not AnyWorkspaceLoaded Then
            ShowFree
            ' If no workspace has been opened, the user can only
            ' create a new workspace - no further processing
            Exit Sub
        Else
            ' Else determine the workspace that the new object is
            ' being created in
            lngWorkspaceId = WorkspaceSelected
        End If
    End If
    
    Call CheckParentValid(intNodeType)
    
    Select Case intNodeType
        Case gintWorkspace
            Call ShowWorkspaceNew
        ' Pass the workspace identifier as a parameter to the
        ' remaining create functions
        Case gintParameter, gintParameterLabel
            Call ShowCreateParameterForm(gintParameterGeneric, lngWorkspaceId)
        Case gintNodeParamExtension, gintParamExtensionLabel
            Call ShowCreateParameterForm(gintParameterApplication, lngWorkspaceId)
        Case gintNodeParamBuiltIn, gintParamBuiltInLabel
            ShowError errBuiltInUpdateOnly
        Case gintNodeParamConnection, gintParamConnectionLabel
            Call ShowCreateConnectionForm(lngWorkspaceId)
        Case gintConnectionDtl, gintConnDtlLabel
            Call ShowCreateConnDtlForm(lngWorkspaceId)
        Case gintGlobalStep, gintGlobalsLabel, gintGlobalStepLabel
            ' If the user selects the "Globals" label, show the
            ' form to create a global step
            ' A common procedure displays the step properties form
            ' to create the different types of steps. Pass the
            ' type of step being created as a parameter, so that
            ' it can customize the form for each step type
            Call ShowCreateStepForm(gintGlobalStep, lngWorkspaceId)
        Case gintManagerStep, gintWorkerStep
            ' A common procedure displays the step properties form
            ' to create the different types of steps. Pass the
            ' type of step being created as a parameter, so that
            ' it can customize the form for each step type
            Call ShowCreateStepForm(intNodeType, lngWorkspaceId, cStepRec)
        Case Else
            ' Do nothing
            BugMessage "Invalid node type passed to NewNode!"
    End Select
    
    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
NewNodeErr:
    ' Log the VB error code and display a more friendly
    ' error message to the user
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    gstrSource = mstrModuleName & "NewNode"
    Call ShowError(errUnableToCreateNewObject)

End Sub
Public Sub PopupNewNode(ByVal intIndex As Integer)
    ' Called when the user selects Create on the popup menu
    ' The index that the user has selected is passed in

    Dim intStepType As Integer
    
    On Error GoTo PopupNewNodeErr
    
    ' If the user has selected the Create option, we need to
    ' create a node based on the type of node he was on, when
    ' he selected the Create
    ' If he selects the Manager/Worker option, we know what
    ' type of node to create!
    If intIndex = 0 Then
        If frmMain.mnuPopupNew(intIndex).Caption = mstrCreateManager Then
            Call NewNode(gintManagerStep)
        Else
            Call CreateNewNode
        End If
    Else
        Call NewNode(intIndex)
    End If

    Exit Sub
    
PopupNewNodeErr:
    ' Log the VB error code and display a more friendly
    ' error message to the user
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    gstrSource = mstrModuleName & "PopupNewNode"
    Call ShowError(errUnableToCreateNewObject)

End Sub
Public Sub SetKeywordText(txtText As TextBox, _
        Optional ByVal strText As String, _
        Optional ByVal strTag As String = gstrEmptyString)

    On Error GoTo SetKeywordTextErr

    Call SetAppKeywordColours(txtText)
    
    ' Set the tag and text for the passed in control
    If Not StringEmpty(strTag) Then
        txtText.Tag = strTag
    End If
    
    If Not StringEmpty(strText) Then
        txtText.Text = strText
    End If
    
    Exit Sub

SetKeywordTextErr:
    LogErrors Errors
    Resume Next

End Sub
Public Sub EnableStop(blnEnabled As Boolean)
    
    ' Enables/disables the Stop menu command
    On Error GoTo EnableStopErr
    
    frmMain.mnuFileStop.Enabled = blnEnabled
    frmMain.tbToolBar.Buttons(tbStop).Enabled = blnEnabled
    
    Exit Sub

EnableStopErr:
    
    Call LogErrors(Errors)
    Resume Next
    
End Sub
Public Function AnyWorkspaceLoaded() As Boolean

    AnyWorkspaceLoaded = frmMain.treStepView.Nodes.Count > 0
    
End Function
Public Sub OpenDatabaseFile()
    
    On Error GoTo OpenDatabaseFileErr
    
    ' Close all open workspaces
    'CloseOpenWorkspaces
    
    SMOpenDatabase
    
    Exit Sub
    
OpenDatabaseFileErr:
    ' Log the VB error code
    LogErrors Errors
    gstrSource = mstrModuleName & "OpenDatabaseFile"
    ShowError errOpenDbFailed

End Sub

Public Sub WspModified(lngWorkspaceId As Long)

    On Error GoTo WspModifiedErr
    
    frmMain.treStepView.Nodes(MakeKeyValid(lngWorkspaceId, gintWorkspace)).Tag = mstrDirty
    
    Exit Sub
    
WspModifiedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Show an error to the user
    ShowError errSaveChanges
    
End Sub
Public Function IsWspModified(strWspName As String) As Boolean

    On Error GoTo IsWspModifiedErr
    
    IsWspModified = frmMain.treStepView.Nodes(strWspName).Tag = mstrDirty
    
    Exit Function
    
IsWspModifiedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Show an error to the user
    ShowError errSaveChanges
    
End Function

Public Sub WspCommitted(lngWorkspaceId As Long)

    On Error GoTo WspCommittedErr
    
    frmMain.treStepView.Nodes( _
            MakeKeyValid(lngWorkspaceId, gintWorkspace)).Tag _
            = mstrClean
    
    Exit Sub
    
WspCommittedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Show an error to the user
    ShowError errSaveChanges
    
End Sub

Public Function WorkspaceSelected(Optional strNodeKey As String) As Long

    ' If a node key has been passed in, Returns the id for the
    ' workspace that the key belongs to.
    ' Else, it returns the workspace id corresponding to the node
    ' selected by the user e.g. if the user has selected a parameter
    ' belonging to the workspace "A" with workspace id, 10, it
    ' returns the workspace id 10
    ' A return value of 0 means that no node was selected by the
    ' user
    
    Dim lngWorkspaceId As String
    Dim strSelectedNodeKey As String
    Dim nodRootNode As Node
    Dim nodAnyNode As Node
    
    On Error GoTo WorkspaceSelectedErr
    gstrSource = mstrModuleName & "WorkspaceSelected"
    
    ' Initialize the return value of the function to an error
    WorkspaceSelected = 0
    
    ' First check if a node key has been passed to the function
    If StringEmpty(strNodeKey) Then
        ' Check if a node was selected by the user
        strSelectedNodeKey = ItemSelectedInTree(frmMain.treStepView)
    Else
        strSelectedNodeKey = strNodeKey
    End If
        
    ' The workspace node is the top level node in the tree
    ' hierarchy.
    
    If StringEmpty(strSelectedNodeKey) Then
        ' If the user has not selected a node, find the
        ' workspace id for the root node in the tree
        Set nodRootNode = GetRootNode(frmMain.treStepView)
        If nodRootNode Is Nothing Then
            On Error GoTo 0
            Err.Raise vbObjectError + errNoWorkspaceLoaded, _
                    gstrSource, _
                    LoadResString(errNoWorkspaceLoaded)
        End If
    Else
        Set nodRootNode = frmMain.treStepView.Nodes(strSelectedNodeKey)
        ' We need to loop till we hit a workspace node since there
        ' is no property that gives us the top level node for a
        ' given node (The root property always points to the first
        ' top level node added to the tree)
        While InStr(nodRootNode.Key, mstrWorkspacePrefix) = 0
            If Not nodRootNode.Parent Is Nothing Then
                Set nodRootNode = nodRootNode.Parent
            Else
                On Error GoTo 0
                Err.Raise vbObjectError + errUnableToGetWorkspace, _
                        gstrSource, _
                        LoadResString(errUnableToGetWorkspace)
            End If
        Wend
    End If
    
    ' The top-level node displays the workspace name to the user
    ' Call a procedure to retrieve the workspace id
    WorkspaceSelected = MakeIdentifierValid(nodRootNode.Key)
    
    Exit Function

WorkspaceSelectedErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errWorkspaceSelectedFailed, _
            gstrSource, _
            LoadResString(errWorkspaceSelectedFailed)

End Function
Public Function IdentifierSelected() As Long

    ' Returns the identifier for the tree view node selected by the user.
    ' If none has been selected, returns 0
    
    Dim strSelectedNodeKey As String
    
    On Error GoTo IdentifierSelectedErr
    
    strSelectedNodeKey = ItemSelectedInTree(frmMain.treStepView)
    
    IdentifierSelected = IIf(StringEmpty(strSelectedNodeKey), 0, MakeIdentifierValid(strSelectedNodeKey))
    
    Exit Function

IdentifierSelectedErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errIdentifierSelectedFailed, _
            gstrSource, _
            LoadResString(errIdentifierSelectedFailed)

End Function


