Attribute VB_Name = "EditNode"
'  FILE:       EditNode.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module contains the procedures for the different
'              edit operations that can be carried out in a workspace
'              by the user
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "EditNode."

Private blnCopy As Boolean
Private Function CopyStep(ByVal NodeIdentifier As Long, _
        ByVal NodeKey As String, ByVal bCopy As Boolean) As Object

    ' Returns an object populated with the data for the passed in Step identifier
    
    On Error GoTo CopyStepErr
    gstrSource = mstrModuleName & "CopyStep"

    If bCopy And frmMain.treStepView.Nodes(NodeKey).Children > 0 Then
        ShowError errBranchWillMove
    End If
    
    Set CopyStep = gcSteps.QueryStep(NodeIdentifier)
    Exit Function
    
CopyStepErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Set CopyStep = Nothing
    Err.Raise vbObjectError + errCopyStepFailed, gstrSource, _
            LoadResString(errCopyStepFailed)
    
End Function
Private Function BranchMoved(ByVal objObject As Object, ByVal intNodeType As Integer) As Boolean
    If (intNodeType = gintManagerStep) Then
        If frmMain.treStepView.Nodes(MakeKeyValid(objObject.StepId, gintManagerStep)).Children > 0 Then
            BranchMoved = True
        Else
            BranchMoved = False
        End If
    Else
        BranchMoved = False
    End If
End Function

Public Function CopyNode(ByRef strFrom As String, _
        ByVal bCopy As Boolean, Optional ByVal strNode As String = gstrEmptyString) _
        As Object
    ' This procedure handles the processing when the user
    ' copies a node - it returns an object containing the data
    ' for the node copied by the user and initializes the passed
    ' in string to the parent of the node being copied
    ' This procedure never raises an error - so use the returned
    ' object to see if it has valid node data
    
    Dim objCopiedNode As Object
    Dim lngNodeIdentifier As Long
    Dim strNodeKey As String

    On Error GoTo CopyNodeErr

    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If StringEmpty(strNode) Then
        ' Call a procedure to determine the type of node selected
        ' by the user and call the appropriate delete function
        strNodeKey = ItemSelectedInTree(frmMain.treStepView)
    Else
        strNodeKey = strNode
    End If
    lngNodeIdentifier = MakeIdentifierValid(strNodeKey)
    
    Select Case TypeOfNode(strNodeKey)
        Case gintParameter, gintNodeParamExtension
            Set objCopiedNode = CopyParameter(lngNodeIdentifier, strNodeKey)
            
        Case gintNodeParamConnection
            Set objCopiedNode = CopyConnection(lngNodeIdentifier, strNodeKey)
            
        Case gintConnectionDtl
            Set objCopiedNode = CopyConnDtl(lngNodeIdentifier, strNodeKey)
            
        Case gintNodeParamBuiltIn
            ShowError errBuiltInUpdateOnly
        
        Case gintGlobalStep, gintManagerStep, gintWorkerStep
            Set objCopiedNode = CopyStep(lngNodeIdentifier, strNodeKey, bCopy)
            
        Case gintWorkspace
            ' Do not allow the user to copy a workspace at present
'            Set objCopiedNode = CopyWorkspace( _
'                    lngNodeIdentifier, strNodeKey)
            Exit Function
        Case Else
            ' Do nothing! The label nodes will fall through
            ' to this case
            Exit Function
    End Select
    
    Set CopyNode = objCopiedNode
    Set objCopiedNode = Nothing
    If Not frmMain.treStepView.Nodes(strNodeKey).Parent Is Nothing Then
        strFrom = frmMain.treStepView.Nodes(strNodeKey).Parent.Key
    Else
        strFrom = gstrEmptyString
    End If
    
    ' Set an indicator that the node has been copied
    blnCopy = True
    
    ToggleCutAndPaste True
    
    ' Reset the mousepointer
    Call ShowFree
    Exit Function
    
CopyNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Set CopyNode = objCopiedNode
    ' Reset the mousepointer
    Call ShowFree
    Call ShowError(errCopyNodeFailed)
    
End Function

Public Function CutNode(ByRef strFrom As String) As Object
    ' This procedure handles the processing when the user
    ' cuts a node - it returns an object containing the data
    ' for the node cut by the user  and initializes the passed
    ' in string to the parent of the node being cut

    On Error GoTo CutNodeErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    ' Call a procedure that will populate an object with
    ' the data for the node cut by the user
    Set CutNode = CopyNode(strFrom, False)
    
    ' Provide visual indication that it has been copied
    
    ' Set an indicator that the node has not been copied
    blnCopy = False
    
    ' Reset the mousepointer
    Call ShowFree
    Exit Function
    
CutNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    Call ShowError(errCutNodeFailed)
    
End Function
Public Sub AddNode(treStepView As TreeView, _
    lvwStepView As comctllib.ListView, _
    ByVal lngKey As Long, ByVal strStringToAdd As String, _
    ByVal lngWorkspaceId As Long, ByVal intNodeType As Integer, _
    ByVal objNode As Object, _
    Optional ByVal ParentKey As String = gstrEmptyString, _
    Optional ByVal NodeKey As String = gstrEmptyString, _
    Optional ByVal SiblingKey As String = gstrEmptyString)
    
    ' NOTE: A sibling will always be inserted B4 the passed in sibling key!
    
    Dim nodTreeNode As Node
    Dim intImageIndex As Integer
    
    On Error GoTo AddNodeErr
    
    If StringEmpty(NodeKey) Then
        NodeKey = MakeKeyValid(lngKey, intNodeType, lngWorkspaceId)
    End If
    
    ' Call a procedure to add the node to the tree view
    Set nodTreeNode = AddNodeToTree(treStepView, NodeKey, strStringToAdd, lngWorkspaceId, _
            intNodeType, objNode, ParentKey, SiblingKey)
    
    Call AddNodeToList(treStepView, lvwStepView, nodTreeNode, _
            NodeKey, strStringToAdd, objNode, intNodeType)
    
    Exit Sub
    
AddNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "AddNode"
    Err.Raise vbObjectError + errAddNodeFailed, _
            gstrSource, _
            LoadResString(errAddNodeFailed)

End Sub
Public Sub DeleteViewNode(treStepView As TreeView, _
        lvwStepView As ListView, _
        lngNodeKey As Long, intTypeOfNode As Integer, _
        ByVal lngWorkspace As Long)
    
    ' Removes the node corresponding to the passed in
    ' identifier from the tree and list view controls

    Dim strKey As String
    
    On Error GoTo DeleteViewNodeErr
    
    strKey = MakeKeyValid(lngNodeKey, intTypeOfNode, lngWorkspace)
    
    treStepView.Nodes.Remove strKey
    
    ' Call a procedure to delete the node from the list view
    Call DeleteListNode(lvwStepView, strKey)
    If lvwStepView.Tag = strKey Then
        lvwStepView.Tag = gstrEmptyString
    End If
    
    Exit Sub
    
DeleteViewNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteViewNodeFailed, _
            mstrModuleName & "DeleteViewNode", _
            LoadResString(errDeleteViewNodeFailed)
    
End Sub

Public Function DragNode(nodDragged As Object) As Object
    ' Returns a reference to the node being dragged by the
    ' user, if it supports a drag-drop operation
    
    Dim strDragKey As String
    
    On Error GoTo DragNodeErr
   
    ' Check if the object being dragged is valid - the only
    ' objects that support drag and drop operations are
    ' the nodes in the list and tree view controls
    If TypeOf nodDragged Is ListItem Then
        strDragKey = nodDragged.Key
    
    ElseIf TypeOf nodDragged Is Node Then
        strDragKey = nodDragged.Key
    
    Else
        ' The drag-'n-drop operation is not valid
        Set nodDragged = Nothing
        strDragKey = gstrEmptyString
    End If
        
    If Not StringEmpty(strDragKey) Then
        If IsLabel(strDragKey) Or TypeOfNode(strDragKey) = gintWorkspace Then
            ' Check if the node being dragged is a workspace or
            ' a label node - do not support drag-drop
            Set nodDragged = Nothing
        End If
    End If
    
    Set DragNode = nodDragged
    Exit Function
    
DragNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errDragDropFailed

End Function
Public Sub UpdateView(treStepView As comctllib.TreeView, _
        lvwStepView As comctllib.ListView, _
        lngIdentifier As Long, _
        strNodeText As String, _
        objNodeData As Object, _
        intTypeOfNode As Integer)
    
    ' Calls procedures to update the list and tree views with
    ' the passed in node data
        
    Dim strNodeKey As String
    
    On Error GoTo UpdateViewErr
    
    strNodeKey = MakeKeyValid(lngIdentifier, intTypeOfNode)
    
    ' Call a procedure to update the tree view
    Call UpdateTreeNode(treStepView, strNodeKey, strNodeText, objNodeData, intTypeOfNode)
        
    ' Call a procedure to update the list view
    Call UpdateListNode(treStepView, lvwStepView, strNodeKey, objNodeData, intTypeOfNode)
    
    Exit Sub
    
UpdateViewErr:
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "UpdateView"
    ' Since this is not a critical error, show an error and exit
    ShowError errUpdateViewFailed
        
End Sub

Public Sub PasteNode(ByVal objNode As Object, ByVal strFrom As String)
    ' This procedure handles the processing when the user
    ' pastes a node - the previously cut/copied node has
    ' to be passed in

    On Error GoTo PasteNodeErr
    
    Dim intObjectType As Integer
    
    ' First check if there is a previously cut or copied node
    If objNode Is Nothing Then
        ' Do nothing - safe response
        Exit Sub
    End If
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    ' Call a procedure to determine whether the object
    ' passed is a Step, Workspace or Parameter node
    intObjectType = TypeOfObject(objNode)
    
    ' Call a procedure to create new nodes, depending on the
    ' type of object cut/copied by the user
    Call CreateNode(objNode, intObjectType, strFrom, blnCopy)
        
    ' Check to see if the node has to be cut
    If Not blnCopy And (intObjectType = gintParameter Or _
            intObjectType = gintParameterConnect Or intObjectType = gintWorkspace) Then
        ' Call a procedure to delete the cut node
        Call DeleteNode(objNode)
    End If
    ToggleCutAndPaste False
    
    ' Reset the mousepointer
    Call ShowFree
    Exit Sub
    
PasteNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    Call ShowError(errPasteNodeFailed)
    
End Sub
Public Sub CreateNewNode()

    ' Called when the user selects the New button on the toolbar
    ' or the Create option on the context-sensitive popup menu
    ' Here the type of object that will be created depends on
    ' the node selected by the user
    
    Dim intNodeType As Integer
    Dim sSelNode As String
    Dim cNewStep As cStep
    
    On Error GoTo CreateNewNodeErr
    
    sSelNode = ItemSelectedInTree(frmMain.treStepView)
            
    ' Call a procedure to determine the type of node selected by
    ' the user
    intNodeType = TypeOfNode(sSelNode)
    
    Call CheckParentValid(intNodeType)
    
    If intNodeType = gintManagerStep Or intNodeType = gintWorkerStep Or _
            intNodeType = gintStepLabel Then
        ' Call a procedure that will display the step types form
        ' and return the step type the user wants to create
        intNodeType = GetStepTypeAndPosition(sSelNode)
        If intNodeType = gintNoOption Then
            ' If the user does not specify a step type, just exit
            Exit Sub
        End If
        
        Set cNewStep = New cStep
        ' Find the position of the new node being created
        Call GetStepPositionDtls(cNewStep)
    End If
    
    ' Call a procedure which will call the appropriate create
    ' function based on the Node type parameter passed to it
    Call NewNode(intNodeType, cNewStep)
    Exit Sub
    
CreateNewNodeErr:

    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "CreateNewNode"
    Call ShowError(errCreateNewNodeFailed)
    
End Sub
Private Sub CreateNode(ByVal objObject As Object, ByVal intNodeType As Integer, _
        ByVal strFrom As String, ByVal bCopy As Boolean)
    
    ' Calls the create functions for the different objects
    ' (workspace, parameter, step) passed in based on the
    ' object type
    
    Dim lngWorkspaceId As Long
    
    On Error GoTo CreateNodeErr
    
    If intNodeType <> gintWorkspace Then
        If Not AnyWorkspaceLoaded Then
            ' If no workspace has been opened, the user can only
            ' create a new workspace - no further processing
            Exit Sub
        Else
            ' Else determine the workspace that the new object is
            ' being created in
            lngWorkspaceId = WorkspaceSelected
        End If
    End If
    
    Call CheckParentValid(intNodeType)
    
    Select Case intNodeType
        Case gintWorkspace
            Call CreateWsp(objObject)
            
        ' Pass the workspace identifier as a parameter to the
        ' remaining create functions
        Case gintParameter
            Call CreateParameter(objObject, lngWorkspaceId)
        
        Case gintParameterConnect
            Call CreateConnection(objObject, lngWorkspaceId)
        
        Case gintConnectionDtl
            Call CreateConnDtl(objObject, lngWorkspaceId)
        
        Case gintGlobalStep, gintManagerStep, gintWorkerStep
            ' A common procedure creates the different
            ' types of steps.
            Call CreateStep(objObject, lngWorkspaceId, strFrom, bCopy)
        
        Case Else
            ' Do nothing
            BugMessage "Invalid node type passed to CreateNode!"
    End Select
    
    Exit Sub
    
CreateNodeErr:
    ' Log the VB error code
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCreateNodeFailed, mstrModuleName & "CreateNode", _
            LoadResString(errCreateNodeFailed)
    
End Sub
Public Sub DeleteNode(Optional ByVal NodeToDelete As Object)

    ' Called when the user selects the Delete option on the
    ' menu or toolbar. The type of object that will be deleted
    ' depends on the node selected by the user when the delete
    ' option was selected
    
    Dim lngNodeIdentifier As Long
    Dim strNodeKey As String
    Dim intNodeType As Integer

    On Error GoTo DeleteNodeErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    ' Call a procedure to determine the type of node selected by
    ' the user and call the appropriate delete function

    If NodeToDelete Is Nothing Then
        strNodeKey = ItemSelectedInTree(frmMain.treStepView)
        lngNodeIdentifier = MakeIdentifierValid(strNodeKey)
        intNodeType = TypeOfNode(strNodeKey)
    Else
        intNodeType = TypeOfObject(NodeToDelete)
    End If
    
    Select Case intNodeType
        Case gintWorkspace
            If Not StringEmpty(strNodeKey) Then
                ' Call the workspace delete function only if the workspace
                ' node has been selected by the user
                Call WorkspaceDelete(NodeToDelete, lngNodeIdentifier)
            End If
        
        Case gintParameter, gintNodeParamExtension
            Call ParameterDelete(NodeToDelete, lngNodeIdentifier)
        
        Case gintNodeParamConnection
            Call ConnectionDelete(NodeToDelete, lngNodeIdentifier)
        
        Case gintConnectionDtl
            Call ConnDtlDelete(NodeToDelete, lngNodeIdentifier)
        
        Case gintNodeParamBuiltIn
            ShowError errBuiltInUpdateOnly
            
        Case gintGlobalStep, gintManagerStep, gintWorkerStep
            Call StepDelete(NodeToDelete, strNodeKey, lngNodeIdentifier)
            
        Case Else
            ' Do nothing! The label nodes will fall through
            ' to this case
    End Select
    
    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
DeleteNodeErr:

    ' Log the VB error code and display a more friendly
    ' error message to the user
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    Call ShowError(errDeleteNodeFailed)
    
End Sub
Public Sub DropNode(nodDragged As Object, nodDropped As Object)
    
    Dim strDragKey As String
    Dim objDragNode As Object
    Dim strFrom As String
    Dim intObjectType As Integer
    
    On Error GoTo DropNodeErr
    
    ' Check if the node being dropped to is valid
    If TypeOf nodDropped Is Node Then
        
        ' Set the mousepointer to indicate Busy
        Call ShowBusy
        
        strDragKey = nodDragged.Key
    
        ' Call a procedure that will populate an object with
        ' the data for the node dragged by the user
        Set objDragNode = CopyNode(strFrom, strDragKey)
        If objDragNode Is Nothing Then
            Exit Sub
        End If
        ' Provide visual indication that it has been copied
        
        ' Call a procedure to determine whether the object
        ' passed is a Step, Workspace or Parameter node
        intObjectType = TypeOfObject(objDragNode)
        
        ' Call a procedure to create new nodes, depending on the
        ' type of object cut/copied by the user
        Call CreateNode(objDragNode, intObjectType, strFrom, bCopy:=False)
        
        ' Call a procedure to delete the cut node
        Call DeleteNode(objDragNode)
    
        ' Reset the mousepointer
        Call ShowFree
    
    End If
            
    Exit Sub
    
DropNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errDragDropFailed

End Sub
