Attribute VB_Name = "RunStatus"
'  FILE:       RunStatus.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Updates the run form with the status of all executing steps
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this class
Private Const mstrModuleName As String = ".RunStatus."

' This flag is used to check whether an error has already
' been displayed to the user
Private mblnErrored As Boolean

Private Const mstrRunStart As String = "Started running workspace: "
Private Const mstrRunComp As String = "Completed running workspace: "
Private Const mstrAt As String = " at "
Private Const mstrAtComp As String = ". The run started at "
Private Const mstrRunElapsed As String = "Elapsed time for the run is "
Private Const mstrRunElapsedComp As String = "The run took "
Private Const mintGridHeader As Integer = 0
Private Const mstrHdrRefresh As String = "&Refresh"
Private Const mstrStopRun As String = "&Stop Run"
Private Const mstrClose As String = "&Close"
Private Const mstrViewLog As String = "&View Log"

Private Enum RunColumns
    mintRowNum = 0
    mintLabel
    mintPath
    mintIterator
    mintStepText
    mintStartTimeHidden
    mintStartTime
    mintElapsedTime ' Used as an upper bound by the grid initialization routines
End Enum

Private Type DateSM
    mlngDays As Long
    mlngHours As Long
    mlngMinutes As Long
    mlngSeconds As Long
    mlngMSeconds As Long
End Type

Private Type FormPosition
    lTop As Long
    lLeft As Long
    lHeight As Long
    lWidth As Long
    lColWidth(mintRowNum To mintElapsedTime) As Long
End Type

Private RunFormPosition As FormPosition

Private Const SECONDS_PER_DAY = 86400
Private Const SECONDS_PER_HOUR = SECONDS_PER_DAY / 24
Private Const SECONDS_PER_MINUTE = SECONDS_PER_HOUR / 60

Private Const MSECONDS_PER_SECOND As Long = 1000
Private Const MSECONDS_PER_MINUTE As Long = MSECONDS_PER_SECOND * 60
Private Const MSECONDS_PER_HOUR As Long = MSECONDS_PER_MINUTE * 60
Private Const MSECONDS_PER_DAY As Long = MSECONDS_PER_HOUR * 24

Private Sub CloseRunStatus()
    Dim iTemp As Integer

    On Error GoTo CloseRunStatusErr
    
    ' Any changes to the listview, treeview, etc. to be taken care of here
    
    ' Save the form settings
    RunFormPosition.lHeight = frmRunning.Height
    RunFormPosition.lWidth = frmRunning.Width
    RunFormPosition.lLeft = frmRunning.Left
    RunFormPosition.lTop = frmRunning.Top
    
    For iTemp = LBound(RunFormPosition.lColWidth) To UBound(RunFormPosition.lColWidth)
        RunFormPosition.lColWidth(iTemp) = frmRunning.msSteps.ColWidth(iTemp)
    Next iTemp

    EnableRun (True)
    
    ' Finally unload the run status form
    Unload frmRunning
    
    Exit Sub
    
CloseRunStatusErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, mstrModuleName & "CloseRunStatus"
    
End Sub
Public Sub ConfirmStopRun()
    
    ' Checks if the user really wants to abort all currently
    ' executing transactions and calls the appropriate
    ' procedures.
    
    Dim strMessage As String
    Dim strTitle As String
    Dim lResponse As Long
        
    On Error GoTo ConfirmStopRunErr
    
    If frmRunning.cmdStopRun.Caption = mstrClose Then
        Call CloseRunStatus
        ' Any changes to the listview, treeview, etc. to be
        ' taken care of here
        Unload frmRunning
    Else
        ' Find the message string for the passed in code
        strMessage = Trim$(LoadResString(conStopRun))
        strTitle = Trim$(LoadResString(contStopRun))
        
        ' Display the confirmation message with the Cancel button
        ' set to the default
        lResponse = ShowMessageBox(frmRunning.hWnd, strMessage, strTitle, _
            MB_YESNOCANCEL + MB_ICONQUESTION + MB_APPLMODAL)
                
        Select Case lResponse
            Case IDYES
                ' Disable the Stop command
                ' TODO: Is this needed?
                ' Call EnableStop(False)
                
                ' The user wants to abort all currently executing transactions
                Call frmMain.AbortRun
                
            Case IDNO
                ' Disable the Stop command
                ' TODO: Is this needed?
                ' Call EnableStop(False)
                
                ' Stop execution once all currently executing transactions
                ' are complete
                Call frmMain.StopRun
                
            Case IDCANCEL
                ' Do nothing
                
            Case Else
                BugAssert False, "Invalid response."
                
        End Select
    End If
    
    Exit Sub
    
ConfirmStopRunErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Call ShowError(errConfirmFailed)

End Sub
Public Function GetInstanceStatus(cInstanceRec As cInstance) As String

    ' Returns the status of the instance being executed
    BugAssert Not cInstanceRec Is Nothing, "Instance record invalid"

    If (cInstanceRec.Status < gintDisabled Or cInstanceRec.Status > gintAborted) Then
        cInstanceRec.Status = IIf(cInstanceRec.StartTime = 0, gintPending, _
                IIf(cInstanceRec.EndTime = 0, gintRunning, gintComplete))
    End If
    
    GetInstanceStatus = gsExecutionStatus(cInstanceRec.Status)
    
End Function
Public Function GetInstanceElapsedTime(cInstanceRec As cInstance) As String

    ' Returns the elapsed time for the instance
    
    On Error GoTo GetInstanceElapsedTimeErr

    If cInstanceRec.Status = gintDisabled Or cInstanceRec.StartTime = 0 Then
        GetInstanceElapsedTime = gstrEmptyString
    ElseIf cInstanceRec.EndTime = 0 Then
        GetInstanceElapsedTime = GetElapsedTime(cInstanceRec.StartTime)
    Else
        GetInstanceElapsedTime = ElapsedMsToString(cInstanceRec.ElapsedTime)
    End If
    
    Exit Function
    
GetInstanceElapsedTimeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetInstanceElapsedTime"
    Err.Raise vbObjectError + errProgramError, gstrSource, _
            LoadResString(errProgramError)
    
End Function

Private Sub InitializeRunGrid(msRunGrid As MSFlexGrid)

    On Error GoTo InitializeRunGridErr
    
'    msRunGrid.Cols = 6
    msRunGrid.Cols = mintElapsedTime + 1
    msRunGrid.FixedCols = 0
    
    ' Initialize the fixed row data
    msRunGrid.FixedRows = 1
    ' An empty row since we need to have atleast one non-fixed
    ' row
    msRunGrid.Rows = msRunGrid.FixedRows + 1
    msRunGrid.RowData(msRunGrid.Rows - 1) = 0
    
    ' The first column will display the row number - no header
    ' column header for the field
    ' Note: Additem does not work on a fixed row
    msRunGrid.TextMatrix(mintGridHeader, mintRowNum) = gstrEmptyString
    msRunGrid.TextMatrix(mintGridHeader, mintLabel) = "Step Label"
    msRunGrid.TextMatrix(mintGridHeader, mintPath) = "Step Path"
    msRunGrid.TextMatrix(mintGridHeader, mintIterator) = "Iterator Values"
    msRunGrid.TextMatrix(mintGridHeader, mintStepText) = "Running"
'    msRunGrid.TextMatrix(mintGridHeader, mintStepText) = "Step Text"
'    msRunGrid.TextMatrix(mintGridHeader, mintStepFile) = "Step File"
    msRunGrid.TextMatrix(mintGridHeader, mintStartTime) = "Start Time"
    msRunGrid.TextMatrix(mintGridHeader, mintElapsedTime) = "Elapsed Time"
    
    msRunGrid.RowData(mintGridHeader) = 0
    
    ' Adjust the width of the cells here, if necessary
    msRunGrid.ColWidth(mintRowNum) = 300
    msRunGrid.ColWidth(mintLabel) = msRunGrid.ColWidth(mintLabel)
    msRunGrid.ColWidth(mintPath) = msRunGrid.ColWidth(mintLabel)
    msRunGrid.ColWidth(mintIterator) = msRunGrid.ColWidth(mintLabel)
    msRunGrid.ColWidth(mintStepText) = msRunGrid.ColWidth(mintLabel)
    msRunGrid.ColWidth(mintStartTimeHidden) = 0
    msRunGrid.ColWidth(mintStartTime) = 5.5 * msRunGrid.ColWidth(mintRowNum)
    msRunGrid.ColWidth(mintElapsedTime) = msRunGrid.ColWidth(mintStartTime)
    
    Exit Sub
    
InitializeRunGridErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)

    Resume Next
End Sub
Private Sub InitRunStatusForm(msRunGrid As MSFlexGrid)

    Dim iTemp As Integer

    On Error GoTo InitRunStatusFormErr
    
    If RunFormPosition.lHeight = 0 Then
        ' First time through
        frmRunning.Left = (Screen.Width - frmRunning.Width) / 2
        frmRunning.Top = (Screen.Height - frmRunning.Height) / 2
    Else
        frmRunning.Height = RunFormPosition.lHeight
        frmRunning.Width = RunFormPosition.lWidth
        frmRunning.Left = RunFormPosition.lLeft
        frmRunning.Top = RunFormPosition.lTop
        
        For iTemp = LBound(RunFormPosition.lColWidth) To UBound(RunFormPosition.lColWidth)
            msRunGrid.ColWidth(iTemp) = RunFormPosition.lColWidth(iTemp)
        Next iTemp
    End If

    Exit Sub
    
InitRunStatusFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Resume Next
    
End Sub
Private Function IsEnabled(cStepRecord As cStep) As Boolean
    ' Returns true if the passed in step is enabled
    
    Dim cTempStep As cStep
    Dim cParentStep As cStep
    
    On Error GoTo IsEnabledErr

    IsEnabled = True
    
    If cStepRecord.EnabledFlag Then
        Set cTempStep = cStepRecord
        
        Do While cTempStep.ParentStepId > 0
            Set cParentStep = gcSteps.QueryStep(cTempStep.ParentStepId)
            If Not cParentStep.EnabledFlag Then
                IsEnabled = False
                Exit Do
            End If
            
            Set cTempStep = cParentStep
        Loop
    Else
        IsEnabled = False
    End If
    
    Set cTempStep = Nothing
    Set cParentStep = Nothing
    
    Exit Function
    
IsEnabledErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "IsEnabled"
    Err.Raise vbObjectError + errProgramError, gstrSource, _
            LoadResString(errProgramError)
    
End Function
Private Sub RemoveStepFromGrid(msRunGrid As MSFlexGrid, _
        cStepRecord As cStep, _
        dtmStartTime As Currency, _
        lngInstanceId As Long)

    Dim lngIndex As Long
    Dim lngTemp As Long
    
    On Error GoTo RemoveStepFromGridErr
    
    ' Find the row at which the step appears in the grid
    lngIndex = FindRowInGrid(msRunGrid, lngInstanceId)
    
    If msRunGrid.Rows = msRunGrid.FixedRows + 1 Then
        ' Add an empty row since we need to have atleast one
        ' non-fixed row
        msRunGrid.AddItem gstrEmptyString
    End If
    
    ' Remove the row corresponding to the step from the grid
    msRunGrid.RemoveItem lngIndex
    
    ' Reorder all the remaining rows in the grid
    For lngTemp = lngIndex To msRunGrid.Rows - 1
        msRunGrid.TextMatrix(lngTemp, mintRowNum) = lngTemp
    Next lngTemp
    
    Exit Sub
    
RemoveStepFromGridErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, mstrModuleName & "RemoveStepFromGrid"
    
End Sub
Public Sub ProcessStart(msRunGrid As MSFlexGrid, _
        cStepRecord As cStep, _
        strCommand As String, _
        dtmStartTime As Currency, _
        lngInstanceId As Long)

    Dim lngIndex As Long
    Dim sTemp As String
    Dim cTempStr As New cStringSM
    
    On Error GoTo ProcessStartErr
    
    ' Find the row at which the step appears in the grid
    lngIndex = FindRowInGrid(msRunGrid, lngInstanceId)
    
    ' Replace all special characters before updating the grid with the command string
    sTemp = strCommand
    sTemp = cTempStr.ReplaceSubString(sTemp, vbCrLf, gstrBlank)
    sTemp = cTempStr.ReplaceSubString(sTemp, vbCr, gstrBlank)
    sTemp = cTempStr.ReplaceSubString(sTemp, vbLf, gstrBlank)
    sTemp = cTempStr.ReplaceSubString(sTemp, vbTab, gstrBlank)
    
    msRunGrid.TextMatrix(lngIndex, mintStepText) = sTemp
    
    Exit Sub
    
ProcessStartErr:
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, mstrModuleName & "ProcessStart"
    
End Sub
Private Sub AddStepToGrid(msRunGrid As MSFlexGrid, cStepRecord As cStep, _
        dtmStartTime As Currency, lngInstanceId As Long, sPath As String, sIts As String)

    Dim lngIndex As Long
    
    On Error GoTo AddStepToGridErr
    
    lngIndex = msRunGrid.Rows
    
    If lngIndex = msRunGrid.FixedRows + 1 Then
        ' The single row may either contain step data or it
        ' may be a dummy row
        ' In the latter case, it must be removed, after we
        ' add the step data
        If msRunGrid.RowData(msRunGrid.Rows - 1) = 0 Then
            ' The row is a dummy row - Add the new row in it's place
            lngIndex = lngIndex - 1
        End If
    End If
    
    ' Initialize all the step fields - the elapsed time will be updated later
    msRunGrid.Rows = msRunGrid.Rows + 1
    msRunGrid.TextMatrix(lngIndex, mintRowNum) = lngIndex
    msRunGrid.TextMatrix(lngIndex, mintLabel) = cStepRecord.StepLabel
    msRunGrid.TextMatrix(lngIndex, mintPath) = sPath
    msRunGrid.TextMatrix(lngIndex, mintIterator) = sIts
    msRunGrid.TextMatrix(lngIndex, mintStepText) = cStepRecord.StepText
'    msRunGrid.TextMatrix(lngIndex, mintStepFile) = cStepRecord.StepTextFile
    msRunGrid.TextMatrix(lngIndex, mintStartTimeHidden) = CStr(dtmStartTime)
    msRunGrid.TextMatrix(lngIndex, mintStartTime) = JulianDateToString(dtmStartTime)
    
    ' Initialize the rowdata for the new row to the step identifier
    msRunGrid.RowData(lngIndex) = lngInstanceId
    
    ' Remove the dummy row from the grid
    If msRunGrid.RowData(msRunGrid.Rows - 1) = 0 Then
        msRunGrid.RemoveItem msRunGrid.Rows - 1
    End If
    
    Exit Sub
    
AddStepToGridErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "AddStepToGrid"
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, gstrSource
    
End Sub
Public Sub ResizeGrid()
    ' Resizes the elements on the Run Status form when the form is resized
    Static iHBorder As Integer
    Static iVBorder As Integer
    Static iGridHBorder As Integer
    Static iGridVBorder As Integer
    
    Dim iHMove As Integer
    Dim iVMove As Integer
    Dim iOld As Integer
    Dim iNew As Integer
    
    On Error GoTo ResizeGridErr
    
    If ((iHBorder = 0) Or (iVBorder = 0)) Then
        iHBorder = frmRunning.Width - (frmRunning.cmdStopRun.Left + frmRunning.cmdStopRun.Width)
        iVBorder = frmRunning.Height - (frmRunning.cmdStopRun.Top + frmRunning.cmdStopRun.Height)
        
        iGridHBorder = frmRunning.fraRunSteps.Width - frmRunning.msSteps.Width
        iGridVBorder = frmRunning.fraRunSteps.Height - frmRunning.msSteps.Height
    End If
    
    ' Position the command buttons at the right bottom corner
    ' Also initialize the differential variables
    iOld = frmRunning.cmdStopRun.Left
    iNew = frmRunning.Width - (frmRunning.cmdStopRun.Width + iHBorder)
    frmRunning.cmdStopRun.Left = iNew
    iHMove = iNew - iOld
    iOld = frmRunning.cmdStopRun.Top
    iNew = frmRunning.Height - (frmRunning.cmdStopRun.Height + iVBorder)
    frmRunning.cmdStopRun.Top = iNew
    iVMove = iNew - iOld
    
    frmRunning.cmdRefresh.Left = frmRunning.cmdRefresh.Left + iHMove
    frmRunning.cmdRefresh.Top = frmRunning.cmdRefresh.Top + iVMove
    
    ' Stretch the Workspace details frame horizontally
    frmRunning.fraWorkspace.Width = frmRunning.fraWorkspace.Width + iHMove
    frmRunning.txtWspRunStart.Width = frmRunning.txtWspRunStart.Width + iHMove
    frmRunning.txtWspElapsedTime.Width = frmRunning.txtWspElapsedTime.Width + iHMove
    
    ' Resize the grid to take advantage of the extra/reduced space
    frmRunning.fraRunSteps.Width = frmRunning.fraRunSteps.Width + iHMove
    frmRunning.fraRunSteps.Height = frmRunning.fraRunSteps.Height + iVMove
    frmRunning.msSteps.Width = frmRunning.fraRunSteps.Width - iGridHBorder
    frmRunning.msSteps.Height = frmRunning.fraRunSteps.Height - iGridVBorder
    'frmRunning.msSteps.Height = frmRunning.msSteps.Height + iVMove
    
    Exit Sub
    
ResizeGridErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ResizeGrid"
    WriteError errUpdateDisplayFailed, gstrSource
    
    ShowError errUpdateDisplayFailed
    
End Sub
Private Function GetElapsedTime(dtmStartTime As Currency, _
        Optional dtmEndTime As Currency = gdtmEmpty) As String
        
    ' Determines the elapsed time between the passed in start and end timestamps.
    ' Uses the current time if no end time is specified.
        
    Dim dtElapsed As DateSM
    Dim lngElapsedTime As Currency
    
    On Error GoTo GetElapsedTimeErr
    
    If dtmEndTime = gdtmEmpty Then
        dtmEndTime = Determine64BitTime()
    End If
    
    lngElapsedTime = dtmEndTime - dtmStartTime
    lngElapsedTime = lngElapsedTime * 10000
            
'    dtElapsed.mlngDays = lngElapsedTime \ SECONDS_PER_DAY
'    lngElapsedTime = lngElapsedTime - _
'            (dtElapsed.mlngDays * SECONDS_PER_DAY)
'
    dtElapsed.mlngHours = lngElapsedTime \ MSECONDS_PER_HOUR
    lngElapsedTime = lngElapsedTime - (dtElapsed.mlngHours * MSECONDS_PER_HOUR)

    dtElapsed.mlngMinutes = lngElapsedTime \ MSECONDS_PER_MINUTE
    lngElapsedTime = lngElapsedTime - (dtElapsed.mlngMinutes * MSECONDS_PER_MINUTE)

    dtElapsed.mlngSeconds = lngElapsedTime \ MSECONDS_PER_SECOND
    lngElapsedTime = lngElapsedTime - (dtElapsed.mlngSeconds * MSECONDS_PER_SECOND)
    
    dtElapsed.mlngMSeconds = lngElapsedTime
    
    GetElapsedTime = dtElapsed.mlngHours & gsTimeSeparator & _
            Format$(dtElapsed.mlngMinutes, gsTmFormat) & gsTimeSeparator & _
            Format$(dtElapsed.mlngSeconds, gsTmFormat) & gsMsSeparator & _
            Format$(dtElapsed.mlngMSeconds, gsMSecondFormat)

    Exit Function
    
GetElapsedTimeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "GetElapsedTime"
    On Error GoTo 0
    Err.Raise vbObjectError + errGetElapsedTimeFailed, _
            gstrSource, _
            LoadResString(errGetElapsedTimeFailed)
    
End Function
Private Function ElapsedMsToString(ByVal CurElapsed As Currency) As String
    ' Converts the passed in elapsed time, in milliseconds, to the display format
        
    Dim dtElapsed As DateSM
    Dim ElapsedMs As Currency
    
    On Error GoTo ElapsedMsToStringErr
    
'    dtElapsed.mlngDays = ElapsedMs \ MSECONDS_PER_DAY
'    ElapsedMs = ElapsedMs - _
'            (dtElapsed.mlngDays * MSECONDS_PER_DAY)
'
    ElapsedMs = CurElapsed
    
    dtElapsed.mlngHours = ElapsedMs \ MSECONDS_PER_HOUR
    ElapsedMs = ElapsedMs - (dtElapsed.mlngHours * MSECONDS_PER_HOUR)

    dtElapsed.mlngMinutes = ElapsedMs \ MSECONDS_PER_MINUTE
    ElapsedMs = ElapsedMs - (dtElapsed.mlngMinutes * MSECONDS_PER_MINUTE)

    dtElapsed.mlngSeconds = ElapsedMs \ MSECONDS_PER_SECOND
    ElapsedMs = ElapsedMs - (dtElapsed.mlngSeconds * MSECONDS_PER_SECOND)

    dtElapsed.mlngMSeconds = ElapsedMs
    
    ElapsedMsToString = dtElapsed.mlngHours & gsTimeSeparator & _
            Format$(dtElapsed.mlngMinutes, gsTmFormat) & gsTimeSeparator & _
            Format$(dtElapsed.mlngSeconds, gsTmFormat) & gsMsSeparator & _
            Format$(dtElapsed.mlngMSeconds, gsMSecondFormat)

    Exit Function
    
ElapsedMsToStringErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ElapsedMsToString"
    On Error GoTo 0
    Err.Raise vbObjectError + errGetElapsedTimeFailed, _
            gstrSource, _
            LoadResString(errGetElapsedTimeFailed)
    
End Function

Public Sub RefreshGrid(msRunGrid As MSFlexGrid)

    On Error GoTo RefreshGridErr
    
    If frmRunning.cmdRefresh.Caption = mstrViewLog Then
        Call OpenFile(frmRunning.txtLog)
    Else
        ' Call a procedure to update the elapsed time for all
        ' the running steps and the workspace
        Call UpdateElapsedTime(msRunGrid)
    End If
    
    Exit Sub
    
RefreshGridErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, mstrModuleName & "RefreshGrid"
    
End Sub
Public Sub RunComplete(lngWspId As Long, dtmEndTime As Currency)
    
    On Error GoTo RunCompleteErr

    Call UpdateElapsedTime(frmRunning.msSteps, dtmEndTime = gdtmEmpty, True)
    Call RestoreTreeIcons(lngWspId, frmMain.treStepView)
    
    Call UpdateLabels(False, lngWspId)
    
    ' Allow the user to view results of the run
    frmRunning.cmdRefresh.Caption = mstrViewLog
    frmRunning.cmdStopRun.Caption = mstrClose
    
    ' Disable the Stop command
    Call EnableStop(False)
    
    Exit Sub
    
RunCompleteErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, mstrModuleName & "RunComplete"
    
End Sub
Public Sub RunStart(lngWspId As Long, dtmStartTime As Currency, strWspLog As String)
    
    On Error GoTo RunStartErr

    mblnErrored = False
    
    Call ShowBusy
    
    Load frmRunning
    
    Call UpdateTreeIcons(lngWspId, frmMain.treStepView)
    
    Call InitializeRunGrid(frmRunning.msSteps)
    
    Call InitRunStatusForm(frmRunning.msSteps)
    
    frmRunning.lblWspStartHide = CStr(dtmStartTime)
    Call UpdateLabels(True, lngWspId)
    
    ' Store the workspace id in the form tag - will be used to
    ' determine if the workspace is running
    frmRunning.Tag = CStr(lngWspId)
    frmRunning.txtLog = strWspLog

    frmRunning.cmdRefresh.Caption = mstrHdrRefresh
    frmRunning.cmdStopRun.Caption = mstrStopRun
    
    Call UpdateElapsedTime(frmRunning.msSteps)
    
    ' Enable the Stop command on the main menu
    Call EnableStop(True)
    
    Call ShowFree
    frmRunning.Show ownerform:=frmMain
    
    Exit Sub
    
RunStartErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, mstrModuleName & "RunStart"
    ' Try to get as much done as possible
    Resume Next
    
End Sub
Public Sub StepComplete(cStepRecord As cStep, dtmEndTime As Currency, _
        lngInstanceId As Long)
    
    Dim sKey  As String
    Dim InstanceRec As cInstance
    Dim cStepInstances As cInstances
    Dim lngIndex As Long
    Dim iStatus As InstanceStatus

    On Error GoTo StepCompleteErr
    
    If cStepRecord.StepType <> gintManagerStep Then
        ' Call a procedure to remove the step entry from the grid
        Call RemoveStepFromGrid(frmRunning.msSteps, cStepRecord, dtmEndTime, lngInstanceId)
        
        ' Call a procedure to update the elapsed time for all
        ' remaining steps and the workspace
        Call UpdateElapsedTime(frmRunning.msSteps, dtmEndTime)
    End If

    sKey = MakeKeyValid(cStepRecord.StepId, cStepRecord.StepType)
    Set cStepInstances = frmMain.InstancesForStep(cStepRecord.StepId, iStatus)
    If Not cStepInstances Is Nothing Then
        If cStepInstances.Count = 0 Then
            If iStatus = gintAborted Then
                frmMain.treStepView.Nodes(sKey).Image = gintImageAborted
                frmMain.treStepView.Nodes(sKey).ExpandedImage = gintImageAborted
            End If
        Else
            For lngIndex = 0 To cStepInstances.Count - 1
                Set InstanceRec = cStepInstances(lngIndex)
                If InstanceRec.EndTime = gdtmEmpty Then
                    ' An instance is still executing - let the tree view indicate running
                    Exit Sub
                End If
            Next lngIndex
        End If
    End If
    frmMain.treStepView.Nodes(sKey).Image = gintImageStop
    frmMain.treStepView.Nodes(sKey).ExpandedImage = gintImageStop

    Exit Sub
    
StepCompleteErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, gstrSource
    
End Sub
Public Sub StepStart(cStepRecord As cStep, dtmStartTime As Currency, _
        lngInstanceId As Long, sPath As String, sIts As String)
    Dim sKey As String

    On Error GoTo StepStartErr
    
    If cStepRecord.StepType <> gintManagerStep Then
        ' Call a procedure to add the new step to the grid
        Call AddStepToGrid(frmRunning.msSteps, cStepRecord, dtmStartTime, lngInstanceId, _
                sPath, sIts)
        
        ' Call a procedure to update the elapsed time for all the
        ' running steps and the workspace
        Call UpdateElapsedTime(frmRunning.msSteps, dtmStartTime)
    End If
    
    sKey = MakeKeyValid(cStepRecord.StepId, cStepRecord.StepType)
    frmMain.treStepView.Nodes(sKey).Image = gintImageRun
    frmMain.treStepView.Nodes(sKey).ExpandedImage = gintImageRun

    Exit Sub
    
StepStartErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, gstrSource
    
End Sub
Private Sub UpdateElapsedTime(msStepGrid As MSFlexGrid, Optional ByVal dtmEndTime As Currency = gdtmEmpty, _
        Optional ByVal bRunComplete As Boolean = False)
    ' This procedure updates the elapsed time for all the
    ' fields on the run status form
    Dim strElapsedTime As String
    Dim lngIndex As Long
    
    On Error GoTo UpdateElapsedTimeErr

    ' First update the elapsed time for the workspace
    strElapsedTime = GetElapsedTime(CCur(frmRunning.lblWspStartHide), dtmEndTime)
    If bRunComplete Then
        frmRunning.txtWspElapsedTime = mstrRunElapsedComp & strElapsedTime
    Else
        frmRunning.txtWspElapsedTime = mstrRunElapsed & strElapsedTime
    End If
    
    ' Then update the elapsed time for each of the steps
    ' being displayed in the grid
    For lngIndex = 0 + msStepGrid.FixedRows To msStepGrid.Rows - 1
        If msStepGrid.RowData(lngIndex) > 0 Then
            msStepGrid.TextMatrix(lngIndex, mintElapsedTime) = GetElapsedTime( _
                    CCur(msStepGrid.TextMatrix(lngIndex, mintStartTimeHidden)), _
                    dtmEndTime)
'            BugMessage msStepGrid.TextMatrix(lngIndex, mintLabel) & " is currently running."
        End If
    Next lngIndex
    
    Exit Sub
    
UpdateElapsedTimeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Display an error only if one hasn't been shown already
'    If Not mblnErrored Then
'        ShowError errUpdateDisplayFailed, gstrSource
'    End If
    mblnErrored = True
    
    ' Write an error to the log in any case
    WriteError errUpdateDisplayFailed, mstrModuleName & "UpdateElapsedTime"
    
End Sub
Private Sub UpdateLabels(bStart As Boolean, lngWspId As Long)

    ' Updates the Run Status form labels depending on the run status
    
    On Error Resume Next
    
    If bStart Then
        ' Initialize the caption for the start run label
        frmRunning.txtWspRunStart = mstrRunStart & _
                CStr(GetWorkspaceDetails(WorkspaceId:=lngWspId)) & _
                mstrAt & JulianDateToString(frmRunning.lblWspStartHide)
    Else
        frmRunning.txtWspRunStart = mstrRunComp & _
                CStr(GetWorkspaceDetails(WorkspaceId:=lngWspId)) & _
                mstrAtComp & JulianDateToString(frmRunning.lblWspStartHide)
    End If

End Sub

Public Function WorkspaceRunning(lngWspId As Long) As Boolean
    ' Returns True if the passed in workspace is currently running
    WorkspaceRunning = IIf(frmRunning.Visible, Trim$(frmRunning.Tag) = Trim$(CStr(lngWspId)), _
            False)
End Function
Public Sub LoadInstancesForStep(treExplore As comctllib.TreeView, _
        lvwExplore As comctllib.ListView, _
        lngStepId As Long)
    ' Loads all the instance records for the step in the
    ' list view
    
    Dim cStepInstances As cInstances
    Dim InstanceRec As cInstance
    Dim cStepRecord As cStep
    Dim intNodeType As Integer
    Dim lngIndex As Long
    Dim strNodeKey As String
    Dim iStatus As InstanceStatus
    
    On Error GoTo LoadInstancesForStepErr

    Set cStepRecord = gcSteps.QueryStep(lngStepId)
    If IsEnabled(cStepRecord) Then

        Set cStepInstances = frmMain.InstancesForStep(lngStepId, iStatus)
        If cStepInstances Is Nothing Then
            ' The run must've just ended
            intNodeType = cStepRecord.StepType
            
            Call AddListNode(treExplore, lvwExplore, _
                    MakeKeyValid(lngStepId, cStepRecord.StepType), _
                    intNodeType, cStepRecord)
        Else
            If cStepInstances.Count = 0 Then
                If iStatus <> gintAborted Then iStatus = gintPending
                Call LoadDummyInstance(treExplore, lvwExplore, cStepRecord, iStatus)
            Else
                For lngIndex = 0 To cStepInstances.Count - 1
                    Set InstanceRec = cStepInstances(lngIndex)
                
                    intNodeType = IIf(cStepRecord.StepType = gintManagerStep, _
                            gintRunManager, gintRunWorker)
                    strNodeKey = MakeKeyValid(InstanceRec.Step.StepId, _
                            intNodeType, _
                            InstanceId:=InstanceRec.InstanceId)
                    Call AddListNode(treExplore, lvwExplore, _
                            InstanceRec.Key, intNodeType, InstanceRec, _
                            ListNodeKey:=strNodeKey)
                Next lngIndex
            End If
        End If
    Else
        ' An instance won't be created for the step -
        ' load a dummy instance with a status of Disabled
        Call LoadDummyInstance(treExplore, lvwExplore, _
                cStepRecord, gintDisabled)
    End If

    Exit Sub
    
LoadInstancesForStepErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "LoadInstancesForStep"
    Err.Raise vbObjectError + errProgramError, gstrSource, _
            LoadResString(errProgramError)
    
End Sub
Public Sub UpdateTreeIcons(ByVal lWspId As Long, treExplore As comctllib.TreeView)
    ' Updates the tree view icons to indicate the run status
    
    Dim cStepInstances As cInstances
    Dim cStepRecord As cStep
    Dim cRunTree As cNavTree
    Dim sKey As String
    Dim lStepId As Long
    Dim iStatus As InstanceStatus
    
    On Error GoTo UpdateTreeIconsErr
    
    Set cRunTree = New cNavTree
    Set cRunTree.Tree = treExplore
    cRunTree.Direction = gintTopDown
    cRunTree.RootNode = LabelStep(lWspId)

    Do
        sKey = cRunTree.NextNode
        If StringEmpty(sKey) Then
            Exit Do
        End If
        
        lStepId = MakeIdentifierValid(sKey)
        Set cStepRecord = gcSteps.QueryStep(lStepId)
        If IsEnabled(cStepRecord) Then
            Set cStepInstances = frmMain.InstancesForStep(lStepId, iStatus)
            If cStepInstances Is Nothing Then
                Call RestoreTreeIcons(lWspId, treExplore)
            Else
                If cStepInstances.Count = 0 Then
                    If iStatus = gintAborted Then
                        treExplore.Nodes(sKey).Image = gintImageAborted
                    Else
                        ' An instance hasn't been created for the step yet - it's pending
                        treExplore.Nodes(sKey).Image = gintImagePending
                    End If
                Else
                    ' The step is running
                    treExplore.Nodes(sKey).Image = gintImageRun
                End If
            End If
        Else
            ' An instance won't be created for the step -
            ' load a dummy instance with a status of Disabled
            treExplore.Nodes(sKey).Image = gintImageDisabled
        End If
        treExplore.Nodes(sKey).ExpandedImage = treExplore.Nodes(sKey).Image
    Loop While Not StringEmpty(sKey)
    Set cRunTree = Nothing

    Exit Sub
    
UpdateTreeIconsErr:
    ' Log the error code raised by Visual Basic -  not a critical error - just proceed
    Call LogErrors(Errors)
    
End Sub
Public Sub RestoreTreeIcons(ByVal lWspId As Long, treExplore As comctllib.TreeView)
    ' Restores the tree view icons
    
    Dim cRunTree As cNavTree
    Dim sKey As String
    Dim lStepId As Long
    Dim iNodeType As Integer
    Dim cStepRec As cStep
    
    On Error GoTo RestoreTreeIconsErr
    
    Set cRunTree = New cNavTree
    Set cRunTree.Tree = treExplore
    cRunTree.Direction = gintTopDown
    cRunTree.RootNode = LabelStep(lWspId)

    Do
        sKey = cRunTree.NextNode
        If StringEmpty(sKey) Then
            Exit Do
        End If
        
        Set cStepRec = gcSteps.QueryStep(MakeIdentifierValid(sKey))
        iNodeType = TypeOfNode(sKey)
        treExplore.Nodes(sKey).Image = GetCollapsedIcon(iNodeType, cStepRec)
        treExplore.Nodes(sKey).ExpandedImage = GetExpandedIcon(iNodeType, cStepRec)
    Loop While Not StringEmpty(sKey)
    
    Set cStepRec = Nothing
    Set cRunTree = Nothing

    Exit Sub
    
RestoreTreeIconsErr:
    ' Log the error code raised by Visual Basic -  not a critical error - just proceed
    Call LogErrors(Errors)
    
End Sub

Private Sub LoadDummyInstance(treExplore As comctllib.TreeView, _
        lvwExplore As comctllib.ListView, _
        cStepRecord As cStep, _
        intStatus As InstanceStatus)
    ' Creates a dummy instance for the passed in step and loads
    ' it in the list view
    
    Dim InstanceRec As cInstance
    Dim intNodeType As Integer
    
    On Error GoTo LoadDummyInstanceErr

    Set InstanceRec = New cInstance
    Set InstanceRec.Step = cStepRecord
    InstanceRec.StartTime = gdtmEmpty
    InstanceRec.EndTime = gdtmEmpty
    InstanceRec.Key = MakeKeyValid(cStepRecord.StepId, _
            cStepRecord.StepType)
    InstanceRec.Status = intStatus
    
    intNodeType = IIf(cStepRecord.StepType = gintManagerStep, _
            gintRunManager, gintRunWorker)
            
    Call AddListNode(treExplore, lvwExplore, InstanceRec.Key, _
            intNodeType, InstanceRec)

    Set InstanceRec = Nothing
    
    Exit Sub
    
LoadDummyInstanceErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "LoadDummyInstance"
    Err.Raise vbObjectError + errProgramError, gstrSource, _
            LoadResString(errProgramError)
    
End Sub
