Attribute VB_Name = "StepView"
'  FILE:       StepView.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module contains the U/I functions for step maintenance
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "StepView."

Private Const msStartIn As String = "Start &In"
Private Const msConnect As String = "Connect&ion"
Public Sub EnableAll(Optional ByVal bEnabled As Boolean = True)

    ' Called when the user selects the EnableAll or DisableAll options -
    ' enables(disables) the selected step and all it's substeps
    
    Dim strKey As String
    Dim cExpTree As cNavTree
    Dim cUpdateStep As cStep
    Dim lngStepId As Long
    
    On Error GoTo EnableAllErr
    
    strKey = ItemSelectedInTree(frmMain.treStepView)
    
    If Not StringEmpty(strKey) Then
        If IsManager(StepKey:=strKey) Then
            Set cExpTree = New cNavTree
            
            Set cExpTree.Tree = frmMain.treStepView
            cExpTree.RootNode = strKey
            cExpTree.Direction = gintTopDown
                    
            Do While (Not StringEmpty(strKey))
                lngStepId = MakeIdentifierValid(strKey)
                
                ' Else query the step array for the
                Set cUpdateStep = gcSteps.QueryStep(lngStepId)
                
                If cUpdateStep.EnabledFlag <> bEnabled Then
                    ' Update the enabled flag
                    cUpdateStep.EnabledFlag = bEnabled
                    Call UpdateStep(cUpdateStep)
                End If
                strKey = cExpTree.NextNode
            Loop
            
            Set cExpTree = Nothing
        End If
    End If
    
    Exit Sub
    
EnableAllErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Call ShowError(errSetEnabledForStepFailed)
    
End Sub
Private Sub PopulateConnections(cboList As ComboBox, lWsp As Long, _
        Optional sDefault As String = gstrEmptyString)

    ' Populates the passed in combo box with all connect strings that have been
    ' defined for the workspace
    Dim lIndex As Long
    
    On Error GoTo PopulateConnectionsErr
        
    cboList.Clear
    
    For lIndex = 0 To gcConnDtls.Count - 1
        If gcConnDtls(lIndex).WorkspaceId = lWsp And _
                gcConnDtls(lIndex).IndOperation <> DeleteOp Then
            
            ' Load the connection string
            cboList.AddItem gcConnDtls(lIndex).ConnName
        End If
    Next lIndex
    
    ' Set the default text to the passed in value
    If StringEmpty(sDefault) Then
        If cboList.ListCount > 0 Then cboList.ListIndex = 0
    Else
        cboList.Text = sDefault
    End If

    Exit Sub
    
PopulateConnectionsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errCustomizeStepFormFailed, mstrModuleName, _
            LoadResString(errCustomizeStepFormFailed)
    
End Sub

Public Sub SetEnabledForStep(ByVal Item As comctllib.ListItem)
    ' Toggles the enabled property for the selected step
    
    Dim cUpdateStep As cStep
    Dim lngStepId As Long
    
    On Error GoTo SetEnabledForStepErr
    
    lngStepId = MakeIdentifierValid(Item.Key)
    
    ' Else query the step array for the
    Set cUpdateStep = gcSteps.QueryStep(lngStepId)
    
    ' Toggle the enabled flag and call a procedure to carry
    ' out the update
    cUpdateStep.EnabledFlag = Not cUpdateStep.EnabledFlag
    Call UpdateStep(cUpdateStep)

    Set cUpdateStep = Nothing
    
    Exit Sub
    
SetEnabledForStepErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errSetEnabledForStepFailed, _
            mstrModuleName & "SetEnabledForStep", _
            LoadResString(errSetEnabledForStepFailed)

End Sub
'Public Sub ListViewCheckToggled(ByVal Item As comctllib.ListItem)
'    ' Called when the enabled property for a step is toggled on/off
'    ' compare the check box state against the enabled flag which is stored
'    ' in the tag property in each node. If the values are different, calls a
'    ' procedure to toggle the enabled property for the node.
'
'    On Error GoTo ListViewCheckToggledErr
'
'    If CBool(Item.Tag) <> Item.Checked Then
'
'        ' Since the enabled property has changed, call a procedure
'        ' to update it in the array
'        Call SetEnabledForStep(Item)
'
'    End If
'
'    Exit Sub
'
'ListViewCheckToggledErr:
'    ' Log the error code raised by Visual Basic
'    Call LogErrors(Errors)
'    gstrSource = mstrModuleName & "ListViewCheckToggled"
'    Call ShowError(errSetEnabledForStepFailed)
'
'End Sub
'
Public Function CircularReference(sTo As String, sFrom As String) As Boolean
    Dim sParent As String
    
    CircularReference = False
    sParent = sTo
    
    While frmMain.treStepView.Nodes(sParent).Parent.Key <> frmMain.treStepView.Nodes(sTo).Root.Key
        sParent = frmMain.treStepView.Nodes(sParent).Parent.Key
        If sParent = sFrom Then
            CircularReference = True
            Exit Function
        End If
    Wend
    
    Exit Function
    
CopyClassPropertiesToFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCopyPropertiesToFormFailed, _
            gstrSource, _
            LoadResString(errCopyPropertiesToFormFailed)

End Function


Private Sub PreviewFile(cStepToCopy As cStep)
    
    Dim sFileName As String
    
    On Error GoTo PreviewFileErr
    
    sFileName = SubstituteParameters(cStepToCopy.StepTextFile, _
            cStepToCopy.WorkspaceId)
                    
    Call PreviewStepTextFile(sFileName, frmSteps.txtStepText, frmSteps.lblStepText)
        
    Exit Sub
    
PreviewFileErr:
    frmSteps.txtStepText.Text = gstrEmptyString
    frmSteps.lblStepText.Caption = "The &Step to be Executed"
    
    ' Log the error code raised by Visual Basic
    gstrSource = mstrModuleName & "PreviewFile"
    Call LogErrors(Errors)
    ' Display a message saying that the preview cannot be
    ' displayed
    Call ShowError(errUnableToPreviewFile)
    
    ' Make sure that the user can only enter a file name
    Call EnteringStepFile
    
End Sub

Public Sub SelectExecutionMethod(intExecMthd As Integer, _
        Optional sConnect As String = gstrEmptyString)
    ' Called when the user selects the Execution method
    ' Enables/disables the start directory field if the execution method is/not ODBC

    On Error GoTo SelectExecutionMethodErr
    
    If intExecMthd = gintExecuteODBC Then
        frmSteps.fraStartDirectory.Caption = msConnect
        
        ' Display a dropdown list of all available connection strings for ODBC
        ' commands
        frmSteps.txtStartDir.Visible = False
        frmSteps.cmdBrowseStartDir.Enabled = False
        frmSteps.cboConnect.Visible = True
        Call PopulateConnections(frmSteps.cboConnect, CLng(frmSteps.lblWorkspaceId), sConnect)
    Else
        frmSteps.fraStartDirectory.Caption = msStartIn
        
        ' Let the user browse for the start directory for Shell commands
        frmSteps.txtStartDir.Visible = True
        frmSteps.cmdBrowseStartDir.Enabled = True
        frmSteps.cboConnect.Visible = False
    End If
    
    Exit Sub
    
SelectExecutionMethodErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errCustomizeStepFormFailed
    
End Sub

Public Sub CopyClassPropertiesToForm(frmSteps As Form, cStepToCopy As cStep)
    
    On Error GoTo CopyClassPropertiesToFormErr
    
    ' Copy the workspace identifier
    frmSteps.lblWorkspaceId = Str(cStepToCopy.WorkspaceId)
    frmSteps.lblStepType = Str(cStepToCopy.StepType)
    frmSteps.lblStepId = Str(cStepToCopy.StepId)
    
    ' Since we use different tabs for the manager and
    ' worker/global steps, initialize different fields on
    ' the form depending on the step type
    If IsManager(StepClass:=cStepToCopy) Then
        ' Copy the step definition properties
        frmSteps.txtMgrStepDescription = cStepToCopy.StepLabel
                
        ' The only other fields on the Manager definition tab
        ' are the enabled flag and the degree of parallelism
        frmSteps.chkMgrEnabled = IIf(cStepToCopy.EnabledFlag, Checked, Unchecked)
        
        ' Important to keep the text and corresponding updown
        ' control values in sync always.
        frmSteps.updMgrNumProcesses.Value = SubstituteParameters(cStepToCopy.DegreeParallelism, cStepToCopy.WorkspaceId)
        frmSteps.txtMgrParallelism = cStepToCopy.DegreeParallelism
    Else
        frmSteps.txtStepDescription = cStepToCopy.StepLabel
    
        If Not StringEmpty(cStepToCopy.StepTextFile) Then
            ' Initialize the tag on the step text field
            frmSteps.txtStepTextFileName.Tag = gstrEntry
            frmSteps.txtStepTextFileName = cStepToCopy.StepTextFile
            
            Call PreviewFile(cStepToCopy)
        Else
            ' Initialize the tag on the step text field
            frmSteps.txtStepTextFileName.Tag = gstrInline
            frmSteps.txtStepText.Tag = gstrEntry
            frmSteps.txtStepText = cStepToCopy.StepText
        End If
        
        If Not cStepToCopy.GlobalFlag Then
            frmSteps.chkEnabled = IIf(cStepToCopy.EnabledFlag, Checked, Unchecked)
            
            ' Important to keep the text and corresponding updown
            ' control values in sync always.
            frmSteps.updNumProcesses.Value = SubstituteParameters(cStepToCopy.DegreeParallelism, cStepToCopy.WorkspaceId)
            frmSteps.txtParallelism = cStepToCopy.DegreeParallelism
        End If
    End If
    
    ' Copy properties depending on the type of step
    If Not cStepToCopy.GlobalFlag Then
        If cStepToCopy.ContinuationCriteria <> gintNoOption Then
            Call SetOption(frmSteps.optOnFailure, cStepToCopy.ContinuationCriteria)
        End If
    
        ' Initialize class with the iterator name for manager and
        ' worker steps
        frmSteps.txtIterator = cStepToCopy.IteratorName
    End If
    
    ' Initialize form with execution details for the step
    If cStepToCopy.ExecutionMechanism <> gintNoOption Then
        Call SetOption(frmSteps.optExecuteUsing, cStepToCopy.ExecutionMechanism)
        Call SelectExecutionMethod(cStepToCopy.ExecutionMechanism, _
                cStepToCopy.StartDir)
        frmSteps.txtStartDir = cStepToCopy.StartDir
    
        frmSteps.txtFailureDetails = cStepToCopy.FailureDetails
    End If
        
    Exit Sub
    
CopyClassPropertiesToFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCopyPropertiesToFormFailed, _
            mstrModuleName & "CopyClassPropertiesToForm", _
            LoadResString(errCopyPropertiesToFormFailed)

End Sub
Public Function GetParent(Optional ByVal strStepKey As String = gstrEmptyString) _
        As cStep
    
    ' Returns the parent step record for the passed in step key
    ' If no parent exists, it returns nothing
    
    Dim cParentStep As cStep
    Dim lngStepId As Long
    Dim strParentKey As String
    
    On Error GoTo GetParentErr
    
    If StringEmpty(strStepKey) Then
        strParentKey = ItemSelectedInTree(frmMain.treStepView)
    Else
        strParentKey = frmMain.treStepView.Nodes(strStepKey).Parent
    End If
    
    If Not IsLabel(strParentKey) Then
        lngStepId = MakeIdentifierValid(strParentKey)
    
        ' Query the parent step record
        Set cParentStep = gcSteps.QueryStep(lngStepId)
    Else
        Set cParentStep = Nothing
    End If
    
    Set GetParent = cParentStep
    Set cParentStep = Nothing
    
    Exit Function
    
GetParentErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Set GetParent = Nothing
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetParent"
    Err.Raise vbObjectError + errQueryStepFailed, gstrSource, _
            LoadResString(errQueryStepFailed)
    
End Function

