VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cArrConstraints"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:       cArrConstraints.cls
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Implements an array of cConstraint objects.
'              Type-safe wrapper around cNodeCollections.
'              Also contains additional functions that determine all the
'              constraints for a step, all constraints in a workspace,
'              validation functions, etc.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

Private mcarrConstraints As cNodeCollections

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cArrConstraints."
Public Sub SaveWspConstraints(ByVal lngWorkspace As Long)
    ' Calls a procedure to commit all changes to the constraints
    ' in the passed in workspace.
    
    Call mcarrConstraints.Save(lngWorkspace)
    
End Sub
Public Property Set ConstraintDB(vdata As Database)

    Set mcarrConstraints.NodeDB = vdata
    
End Property
Public Property Get ConstraintDB() As Database

    Set ConstraintDB = mcarrConstraints.NodeDB
    
End Property

Public Sub Modify(cConsToUpdate As cConstraint)
    
    ' Modify the constraint record
    Call mcarrConstraints.Modify(cConsToUpdate)

End Sub
Public Sub CreateNewConstraintVersion(ByVal lngStepId As Long, _
        ByVal strNewVersion As String, _
        ByVal strOldVersion As String, _
        ByVal intStepType As Integer)

    ' Does all the processing needed to create new versions of
    ' all the constraints for a given step
    ' It inserts new constraint records in the database with
    ' the new version numbers on them
    ' It also updates the version number on all constraints
    ' for the step in the array to the new version passed in
    ' Since it handles both global and manager/worker steps,
    ' it checks for the step_id or global_step_id fields,
    ' depending on the type of step
    
    Dim lngIndex As Long
    Dim cUpdateConstraint As cConstraint
    
    On Error GoTo CreateNewConstraintVersionErr
    mstrSource = mstrModuleName & "CreateNewConstraintVersion"
    
    ' Update the version/global version on Constraint with the
    ' passed in step/global step id
    For lngIndex = 0 To mcarrConstraints.Count - 1
        Set cUpdateConstraint = mcarrConstraints(lngIndex)
        If intStepType = gintGlobalStep Then
            If cUpdateConstraint.GlobalStepId = lngStepId And _
                    cUpdateConstraint.IndOperation <> DeleteOp Then
                cUpdateConstraint.GlobalVersionNo = strNewVersion
                
                ' Set the operation to indicate an insert
                cUpdateConstraint.IndOperation = InsertOp
            End If
        Else
            If cUpdateConstraint.StepId = lngStepId And _
                    cUpdateConstraint.IndOperation <> DeleteOp Then
                cUpdateConstraint.VersionNo = strNewVersion
                
                ' Set the operation to indicate an insert
                cUpdateConstraint.IndOperation = InsertOp
            End If
        End If
    Next lngIndex
    
    Exit Sub
    
CreateNewConstraintVersionErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "CreateNewConstraintVersion"
    On Error GoTo 0
    Err.Raise vbObjectError + errCreateNewConstraintVersionFailed, _
            mstrSource, _
            LoadResString(errCreateNewConstraintVersionFailed)
        
End Sub
Private Sub Class_Initialize()
    
    Set mcarrConstraints = New cNodeCollections
    BugMessage "cArrConstraints: Initialize event - setting Constraint count to 0"
    
End Sub

Private Sub Class_Terminate()

    Set mcarrConstraints = Nothing
    BugMessage "cArrConstraints: Terminate event triggered"
    
End Sub

Public Sub Add(ByVal cConstraintToAdd As cConstraint)

    Set cConstraintToAdd.NodeDB = mcarrConstraints.NodeDB
    
    ' Retrieve a unique constraint identifier
    cConstraintToAdd.ConstraintId = cConstraintToAdd.NextIdentifier
    
    ' Call a procedure to load the constraint record in the array
    Call mcarrConstraints.Add(cConstraintToAdd)
    
End Sub
Public Sub Delete(ByVal cOldConstraint As cConstraint)

    Dim lngDeleteElement As Long
    Dim cConsToDelete As cConstraint
        
    lngDeleteElement = QueryConstraintIndex(cOldConstraint.ConstraintId)
    Set cConsToDelete = mcarrConstraints(lngDeleteElement)
    
    Call mcarrConstraints.Delete(cConsToDelete.Position)
    
    Set cConsToDelete = Nothing
    
End Sub
Private Function QueryConstraintIndex(lngConstraintId As Long) _
        As Long

    Dim lngIndex As Integer

    ' Find the element in the array to be deleted
    For lngIndex = 0 To mcarrConstraints.Count - 1
    
        ' Note: The constraint id is not a primary key field in
        ' the database - there can be multiple records with the
        ' same constraint_id but for different versions of a step
        ' However, since we'll always load the constraint information
        ' for the latest version of a step, we'll have just one
        ' constraint record with a given constraint_id
        If mcarrConstraints(lngIndex).ConstraintId = lngConstraintId Then
            QueryConstraintIndex = lngIndex
            Exit Function
        End If
    
    Next lngIndex
    
    ' Raise error that Constraint has not been found
    ShowError errConstraintNotFound
    On Error GoTo 0
    Err.Raise vbObjectError + errConstraintNotFound, mstrSource, _
        LoadResString(errConstraintNotFound)
    
End Function

Public Function QueryConstraint(ByVal lngConstraintId As Long) _
    As cConstraint
    
    ' Returns a cConstraint object with the property values
    ' corresponding to the Constraint Identifier, lngConstraintId
    
    Dim lngQueryElement As Long
        
    lngQueryElement = QueryConstraintIndex(lngConstraintId)
    
    ' Set the return value to the queried Constraint
    Set QueryConstraint = mcarrConstraints(lngQueryElement)
    
End Function

Public Sub LoadConstraints(ByVal lngWorkspaceId As Long, rstStepsInWsp As Recordset)
    
    ' Loads the constraints array with all the constraints
    ' for the workspace
    Dim recConstraints As Recordset
    Dim qyCons As DAO.QueryDef
    Dim strSql As String
    Dim dtStart As Date
    
    On Error GoTo LoadConstraintsErr
    mstrSource = mstrModuleName & "LoadConstraints"
    
    If rstStepsInWsp.RecordCount = 0 Then
        Exit Sub
    End If
    
    ' First check if the database object has been set
    If mcarrConstraints.NodeDB Is Nothing Then
        On Error GoTo 0
        Err.Raise vbObjectError + errSetDBBeforeLoad, _
                mstrSource, _
                LoadResString(errSetDBBeforeLoad)
    End If
        
    dtStart = Now
    
    ' Select based on the global step id since there might
    ' be constraints for a global step that run are executed
    ' for the workspace
    ' This method has the advantage that if the steps are queried right, everything else follows
    strSql = "Select a.constraint_id, a.step_id, a.version_no, " & _
        " a.constraint_type, a.global_step_id, a.global_version_no, " & _
        " a.sequence_no, b.workspace_id " & _
        " from step_constraints a, att_steps b " & _
        " where a.global_step_id = b.step_id " & _
        " and a.global_version_no = b.version_no " & _
        " and a.global_step_id = [g_s_id] " & _
        " and a.global_version_no = [g_ver_no] " & _
        " and b.archived_flag = [archived] "

    ' Find the highest X-component of the version number
    strSql = strSql & " AND ( a.step_id = 0 or ( cint( mid( a.version_no, 1, instr( a.version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS d " & _
        " WHERE a.step_id = d.step_id " & _
        " and d.archived_flag = [archived] ) "
        
    ' Find the highest Y-component of the version number for the highest X-component
    strSql = strSql & " AND cint( mid( a.version_no, instr( a.version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) = " & _
        " ( select max( cint( mid( version_no, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) ) " & _
        " from att_steps AS y " & _
        " Where a.step_id = y.step_id " & _
        " AND cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS c " & _
        " WHERE y.step_id = c.step_id " & _
        " and c.archived_flag = [archived] ) ) ) ) "
    
    ' Order the constraints by sequence within a given step
    strSql = strSql & " order by a.sequence_no "
    
    Set qyCons = mcarrConstraints.NodeDB.CreateQueryDef(gstrEmptyString, strSql)
    qyCons.Parameters("archived").Value = False
    
    rstStepsInWsp.MoveFirst
    
    While Not rstStepsInWsp.EOF
            
        If Not (rstStepsInWsp!global_flag) Then
            qyCons.Close
            BugMessage "Query constraints Read + load took: " & CStr(DateDiff("s", dtStart, Now))
            Exit Sub
        End If
        
        qyCons.Parameters("g_s_id").Value = rstStepsInWsp!step_id
        qyCons.Parameters("g_ver_no").Value = rstStepsInWsp!version_no
        
        Set recConstraints = qyCons.OpenRecordset(dbOpenSnapshot)
        
        Call LoadRecordsetInConstraintArray(recConstraints)
        recConstraints.Close
        
        rstStepsInWsp.MoveNext
    Wend
    
    qyCons.Close
    BugMessage "Query constraints Read + load took: " & CStr(DateDiff("s", dtStart, Now))
    
    Exit Sub
    
LoadConstraintsErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "LoadConstraints"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadDataFailed, _
            mstrSource, _
            LoadResString(errLoadDataFailed)
        
End Sub
Public Sub UnloadStepConstraints(ByVal lngStepId As Long)
    
    ' Unloads all the constraints for the workspace from
    ' the constraints array
    
    Dim lngIndex As Long
    
    ' Find all constraints in the array with a matching step id
    ' It is important to step in reverse order through the array,
    ' since we delete constraint records!
    For lngIndex = mcarrConstraints.Count - 1 To 0 Step -1
        If mcarrConstraints(lngIndex).GlobalStepId = lngStepId Then

            ' Unload the constraint from the array
            Call mcarrConstraints.Unload(lngIndex)

        End If
    Next lngIndex
    
End Sub
Public Sub UnloadConstraint(cOldConstraint As cConstraint)
    ' Unloads the constraint from the constraints array
    
    Dim lngDeleteElement As Long
    
    lngDeleteElement = QueryConstraintIndex(cOldConstraint.ConstraintId)
    
    Call mcarrConstraints.Unload(lngDeleteElement)
    
End Sub
Private Sub LoadRecordsetInConstraintArray(ByVal recConstraints As Recordset)
    ' Loads all the constraint records in the passed in
    ' recordset into the array

    Dim cNewConstraint As cConstraint
    
    On Error GoTo LoadRecordsetInConsArrayErr
    mstrSource = mstrModuleName & "LoadRecordsetInConstraintArray"
    
    If recConstraints.RecordCount = 0 Then
        Exit Sub
    End If
    
    recConstraints.MoveFirst
    While Not recConstraints.EOF
        Set cNewConstraint = New cConstraint
        
        ' Initialize Constraint values
        cNewConstraint.ConstraintId = CLng(ErrorOnNullField(recConstraints, "Constraint_id"))
        cNewConstraint.StepId = CLng(ErrorOnNullField(recConstraints, "step_id"))
        cNewConstraint.VersionNo = CStr(ErrorOnNullField(recConstraints, "version_no"))
        
        cNewConstraint.GlobalStepId = CLng(ErrorOnNullField(recConstraints, "global_step_id"))
        cNewConstraint.GlobalVersionNo = CStr(ErrorOnNullField(recConstraints, "global_version_no"))
        cNewConstraint.SequenceNo = CInt(ErrorOnNullField(recConstraints, "sequence_no"))
        
        cNewConstraint.WorkspaceId = CLng(ErrorOnNullField(recConstraints, FLD_ID_WORKSPACE))
        cNewConstraint.ConstraintType = CInt(ErrorOnNullField(recConstraints, "constraint_type"))
        
        ' Add this record to the array of Constraints
        mcarrConstraints.Load cNewConstraint
        
        Set cNewConstraint = Nothing
        recConstraints.MoveNext
    Wend
    
    Exit Sub
    
LoadRecordsetInConsArrayErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "LoadRecordsetInConstraintArray"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadRsInArrayFailed, _
            mstrSource, _
            LoadResString(errLoadRsInArrayFailed)
    
End Sub

Public Function ConstraintsForStep( _
        ByVal lngStepId As Long, _
        ByVal strVersionNo As String, _
        Optional ByVal intConstraintType As ConstraintType = 0, _
        Optional ByVal blnSort As Boolean = True, _
        Optional ByVal blnGlobal As Boolean = False, _
        Optional ByVal blnGlobalConstraintsOnly As Boolean = False) _
        As Variant
    
    ' Returns a variant containing an array of cConstraint objects,
    ' containing all the constraints that have been defined for the
    ' given step. If the Global flag is set to true, the
    ' search will be made for all the constraints that have
    ' a matching global_step_id
    
    Dim lngIndex As Long
    Dim cStepConstraint() As cConstraint
    Dim lngConstraintCount As Long
    Dim cTempConstraint As cConstraint
    
    On Error GoTo ConstraintsForStepErr
    mstrSource = mstrModuleName & "ConstraintsForStep"
    
    lngConstraintCount = 0
    
    ' Find each element in the constraints array
    For lngIndex = 0 To mcarrConstraints.Count - 1
        ' If a constraint type has been specified then check
        ' if the constraint type for the record matches the
        ' passed in type
        Set cTempConstraint = mcarrConstraints(lngIndex)
        If Not blnGlobal Then
            If cTempConstraint.StepId = lngStepId And _
                    cTempConstraint.VersionNo = strVersionNo And _
                    cTempConstraint.IndOperation <> DeleteOp And _
                    (intConstraintType = 0 Or _
                    cTempConstraint.ConstraintType = intConstraintType) Then
                ' We have a matching constraint for the given step
                AddArrayElement cStepConstraint, _
                        cTempConstraint, lngConstraintCount
            End If
        Else
            If cTempConstraint.GlobalStepId = lngStepId And _
                    cTempConstraint.GlobalVersionNo = strVersionNo And _
                    cTempConstraint.IndOperation <> DeleteOp Then
                If blnGlobalConstraintsOnly = False Or _
                        (blnGlobalConstraintsOnly And _
                        cTempConstraint.StepId = 0 And _
                        cTempConstraint.VersionNo = gstrMinVersion) Then

                    ' We have a matching constraint for the global step
                    AddArrayElement cStepConstraint, _
                            cTempConstraint, lngConstraintCount
                End If
            End If
        End If
            
    Next lngIndex
    
    ' Set the return value of the function to the array of
    ' constraints that has been built above
    If lngConstraintCount = 0 Then
        ConstraintsForStep = Empty
    Else
        ConstraintsForStep = cStepConstraint()
    End If
    
    ' Sort the constraints
    If blnSort Then
        Call QuickSort(ConstraintsForStep)
    End If
    
    Exit Function
    
ConstraintsForStepErr:
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errConstraintsForStepFailed, _
            mstrSource, _
            LoadResString(errConstraintsForStepFailed)
        
End Function
Private Sub AddArrayElement(ByRef arrNodes() As cConstraint, _
        ByVal objToAdd As cConstraint, _
        ByRef lngCount As Long)
    ' Adds the passed in object to the array
    
    ' Increase the array dimension and add the object to it
    ReDim Preserve arrNodes(lngCount)
    Set arrNodes(lngCount) = objToAdd
    lngCount = lngCount + 1

End Sub

Public Function ConstraintsForWsp( _
        ByVal lngWorkspaceId As Long, _
        Optional ByVal intConstraintType As Integer = 0, _
        Optional ByVal blnSort As Boolean = True, _
        Optional ByVal blnGlobalConstraintsOnly As Boolean = False) _
        As Variant
    
    ' Returns a variant containing an array of cConstraint objects,
    ' containing all the constraints that have been defined for the
    ' given workspace.
    
    Dim lngIndex As Long
    Dim cWspConstraint() As cConstraint
    Dim lngConstraintCount As Long
    Dim cTempConstraint As cConstraint
    
    On Error GoTo ConstraintsForWspErr
    mstrSource = mstrModuleName & "ConstraintsForWsp"
    
    lngConstraintCount = 0
    
    ' Find each element in the constraints array
    For lngIndex = 0 To mcarrConstraints.Count - 1
        ' If a constraint type has been specified then check
        ' if the constraint type for the record matches the
        ' passed in type
        Set cTempConstraint = mcarrConstraints(lngIndex)
        If cTempConstraint.WorkspaceId = lngWorkspaceId And _
                cTempConstraint.IndOperation <> DeleteOp And _
                (intConstraintType = 0 Or _
                cTempConstraint.ConstraintType = intConstraintType) Then
            
            If blnGlobalConstraintsOnly = False Or _
                    (blnGlobalConstraintsOnly And _
                    cTempConstraint.StepId = 0 And _
                    cTempConstraint.VersionNo = gstrMinVersion) Then
                
                ' We have a matching constraint for the workspace
                AddArrayElement cWspConstraint, _
                        cTempConstraint, lngConstraintCount
            End If
        End If
    Next lngIndex
    
    ' Set the return value of the function to the array of
    ' constraints that has been built above
    If lngConstraintCount = 0 Then
        ConstraintsForWsp = Empty
    Else
        ConstraintsForWsp = cWspConstraint()
    End If
    
    ' Sort the constraints
    If blnSort Then
        Call QuickSort(ConstraintsForWsp)
    End If
    
    Exit Function
    
ConstraintsForWspErr:
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errConstraintsForWspFailed, _
            mstrSource, _
            LoadResString(errConstraintsForWspFailed)
        
End Function
Public Function PreConstraintsForStep( _
        ByVal lngStepId As Long, _
        ByVal strVersionNo As String, _
        Optional ByVal blnSort As Boolean) As Variant
    
    ' Returns a variant containing an array of cConstraint objects,
    ' containing all the pre-execution constraints that have
    ' been defined for the given step_id and version
    
    ' Call a function that will return a variant containing
    ' all the constraints of the passed in type
    PreConstraintsForStep = ConstraintsForStep(lngStepId, _
            strVersionNo, gintPreStep, blnSort)
            
End Function
Public Function PostConstraintsForStep( _
        ByVal lngStepId As Long, _
        ByVal strVersionNo As String, _
        Optional ByVal blnSort As Boolean) As Variant
    
    ' Returns a variant containing an array of cConstraint objects,
    ' containing all the Post-execution constraints that have
    ' been defined for the given step_id and version
    
    ' Call a function that will return a variant containing
    ' all the constraints of the passed in type
    PostConstraintsForStep = ConstraintsForStep(lngStepId, _
            strVersionNo, gintPostStep, blnSort)
            
End Function
Public Function PostConstraintsForWsp( _
        ByVal lngWorkspaceId As Long, _
        Optional ByVal blnSort As Boolean) As Variant
    
    ' Returns a variant containing an array of cConstraint objects,
    ' containing all the Post-execution globals that have
    ' been defined for the workspace
    
    ' Call a function that will return a variant containing
    ' all the constraints of the passed in type
    PostConstraintsForWsp = ConstraintsForWsp(lngWorkspaceId, _
            gintPostStep, blnSort, True)
            
End Function
Public Function PreConstraintsForWsp( _
        ByVal lngWorkspaceId As Long, _
        Optional ByVal blnSort As Boolean) As Variant
    
    ' Returns a variant containing an array of cConstraint objects,
    ' containing all the Pre-execution globals that have
    ' been defined for the workspace
    
    ' Call a function that will return a variant containing
    ' all the constraints of the passed in type
    PreConstraintsForWsp = ConstraintsForWsp(lngWorkspaceId, _
            gintPreStep, blnSort, True)
            
End Function
Public Property Get ConstraintCount() As Long

    ConstraintCount = mcarrConstraints.Count
    
End Property

